@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 #
 #
@@ -23,50 +23,51 @@ sys.path.insert(0, str(project_dir))
 
 # If your documentation needs a minimal Sphinx version, state it here.
 #
-needs_sphinx = '3.0.0'
+needs_sphinx = "3.0.0"
 
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = [
-    'sphinx.ext.autodoc',
-    'sphinx.ext.napoleon',
-    'sphinx_autodoc_typehints',
+    "sphinx.ext.autodoc",
+    "sphinx.ext.napoleon",
+    "sphinx_autodoc_typehints",
     # sphinx_autodoc_typehints must be imported after napoleon to properly work.
     # See https://github.com/agronholm/sphinx-autodoc-typehints/issues/15
-    'jupyter_sphinx',
-    'sphinx.ext.autosummary',
-    'sphinx.ext.viewcode',
-    'sphinx.ext.intersphinx',
-    'sphinxcontrib.images',
-    'sphinx.ext.coverage',
-    'sphinx.ext.mathjax',
-    'sphinxcontrib.bibtex',
-    'sphinx.ext.githubpages',
-    'sphinx.ext.todo',
-    'sphinx_copybutton',
-    'sphinxcontrib.youtube',
+    "jupyter_sphinx",
+    "sphinx.ext.autosummary",
+    "sphinx.ext.viewcode",
+    "sphinx.ext.intersphinx",
+    "sphinxcontrib.images",
+    "sphinx.ext.coverage",
+    "sphinx.ext.mathjax",
+    "sphinxcontrib.bibtex",
+    "sphinx.ext.githubpages",
+    "sphinx.ext.todo",
+    "sphinx_copybutton",
+    "sphinxcontrib.youtube",
     "sphinx_panels",
-    'seed_intersphinx_mapping',
+    "seed_intersphinx_mapping",
     "myst_nb",
     "sphinx_togglebutton",
-
 ]
 
-panels_add_bootstrap_css = False  # for sphinx_panel, use custom css from theme, not bootstrap
+panels_add_bootstrap_css = (
+    False  # for sphinx_panel, use custom css from theme, not bootstrap
+)
 
 # releases_github_path = "zfit/zfit"  # TODO: use releases or similar?
 # releases_document_name = "../CHANGELOG.rst"
 
-# jupyter_execute_notebooks = "force"  # use if needed and cache should be ignored
-jupyter_execute_notebooks = "cache"
-if jupyter_execute_notebooks == "cache":
-    jupyter_cache_path = project_dir.joinpath('docs', '.cache', 'myst-nb')
+# nb_execution_mode = "force"  # use if needed and cache should be ignored
+nb_execution_mode = "cache"
+if nb_execution_mode == "cache":
+    jupyter_cache_path = project_dir.joinpath("docs", ".cache", "myst-nb")
     jupyter_cache_path.mkdir(parents=True, exist_ok=True)
     jupyter_cache = str(jupyter_cache_path)
 
 source_suffix = {
-    '.ipynb': 'myst-nb',
-    '.myst': 'myst-nb',
+    ".ipynb": "myst-nb",
+    ".myst": "myst-nb",
 }
 myst_enable_extensions = [
     "amsmath",
@@ -76,27 +77,32 @@ myst_enable_extensions = [
     "html_image",
 ]
 
+bibtex_bibfiles = [str(project_dir.joinpath("docs", "refs.bib"))]
 
-bibtex_bibfiles = [str(project_dir.joinpath('docs', 'refs.bib'))]
-
-zfit_tutorials_path = project_dir.joinpath('docs', '_tmp', 'zfit-tutorials')
+zfit_tutorials_path = project_dir.joinpath("docs", "_tmp", "zfit-tutorials")
 atexit.register(lambda path=zfit_tutorials_path: shutil.rmtree(path))
 pygit2.clone_repository("https://github.com/zfit/zfit-tutorials", zfit_tutorials_path)
 
-execution_in_temp = True
+zfit_images_path = project_dir.joinpath("docs", "images")
+docs_images_path = project_dir.joinpath("docs", "_static", "images")
+docs_images_path.mkdir(parents=True, exist_ok=True)
+atexit.register(lambda path=docs_images_path: shutil.rmtree(path))
+shutil.copytree(zfit_images_path, docs_images_path, dirs_exist_ok=True)
+
+nb_execution_in_temp = True
 
 # Add any paths that contain templates here, relative to this directory.
-templates_path = ['_templates']
+templates_path = ["_templates"]
 
 # The suffix(es) of source filenames.
 # You can specify multiple suffix as a list of string:
-source_suffix = '.rst'
+source_suffix = ".rst"
 
 # The master toctree document.
-master_doc = 'index'
+master_doc = "index"
 
 # General information about the project.
-project = 'zfit'
+project = "zfit"
 copyright = zfit.__copyright__
 author = zfit.__author__
 
@@ -105,7 +111,7 @@ author = zfit.__author__
 # the built documents.
 #
 # The short X.Y version.
-version = zfit.__version__.split('+')[0]
+version = zfit.__version__.split("+")[0]
 # The full version, including alpha/beta/rc tags.
 release = zfit.__version__
 
@@ -114,18 +120,18 @@ release = zfit.__version__
 #
 # This is also used if you do content translation via gettext catalogs.
 # Usually you set "language" from the command line for these cases.
-language = 'en'
+language = "en"
 
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
 # This patterns also effect to html_static_path and html_extra_path
-exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
+exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
 
 # The name of the Pygments (syntax highlighting) style to use.
-pygments_style = 'sphinx'
+pygments_style = "sphinx"
 
 # Automatically add substitutions to all RST files.
-with open('subst_types.txt') as subst_types:
+with open("subst_types.txt") as subst_types:
     rst_epilog = subst_types.read()
 
 # add whitespaces to the internal commands. Maybe move to preprocessing?
@@ -138,25 +144,25 @@ rst_epilog += f"""
 #
 # .. |@docend| replace:: |wzw|
 # """
-with open(str(project_dir) + '/utils/api/argdocs.yaml') as replfile:
+with open(str(project_dir) + "/utils/api/argdocs.yaml") as replfile:
     replacements = yaml.load(replfile, Loader=yaml.Loader)
 for replacement_key in replacements:
-    rst_epilog += f'''
+    rst_epilog += f"""
 .. |@doc:{replacement_key}| replace:: |wzw|
 
 .. |@docend:{replacement_key}| replace:: |wzw|
-'''
-with open('hyperlinks.txt') as hyperlinks:
+"""
+with open("hyperlinks.txt") as hyperlinks:
     rst_epilog += hyperlinks.read()
 
 # makes the jupyter extension executable
 jupyter_sphinx_thebelab_config = {
     # 'bootstrap': False,
     # 'requestKernel': True,
-    'binderOptions': {
-        'repo': "zfit/zfit-tutorials",
-        'binderUrl': "https://mybinder.org",
-        'repoProvider': "github",
+    "binderOptions": {
+        "repo": "zfit/zfit-tutorials",
+        "binderUrl": "https://mybinder.org",
+        "repoProvider": "github",
     },
 }
 
@@ -194,11 +200,11 @@ typehints_document_rtype = True
 # -- autodoc settings ---------------------------------------------
 
 # also doc __init__ docstrings
-autoclass_content = 'both'
-autodoc_member_order = 'bysource'
+autoclass_content = "both"
+autodoc_member_order = "bysource"
 autodoc_default_options = {
-    'show-inheritance': True,
-    'inherited-members': True,
+    "show-inheritance": True,
+    "inherited-members": True,
 }
 autodoc_inherit_docstrings = False
 
@@ -222,26 +228,28 @@ html_theme = "pydata_sphinx_theme"
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ['_static']
+html_static_path = ["_static"]
 
 # These paths are either relative to html_static_path
 # or fully qualified paths (eg. https://...)
 html_css_files = [
-    'css/custom.css',
+    "css/custom.css",
 ]
 
 # Theme options are theme-specific and customize the look and feel of a
 # theme further.
-html_logo = "images/zfit-logo-light_400x168.png"
 
 html_theme_options = {
+    "logo": {
+        "image_light": "images/zfit-logo_400x168.png",
+        "image_dark": "images/zfit-logo-light_400x168.png",
+    },
     "github_url": "https://github.com/zfit/zfit",
     "use_edit_page_button": True,
     "navigation_depth": 3,
     "search_bar_text": "Search zfit...",
     "navigation_with_keys": True,
     "search_bar_position": "sidebar",
-
     # "repository_url": "https://github.com/zfit/zfit",  # adding jupyter book somehow?
     # "repository_branch": "develop",
     # "path_to_docs": "docs",
@@ -257,7 +265,7 @@ html_context = {
 # -- Options for HTMLHelp output ---------------------------------------
 
 # Output file base name for HTML help builder.
-htmlhelp_basename = 'zfitdoc'
+htmlhelp_basename = "zfitdoc"
 
 # -- Options for LaTeX output ------------------------------------------
 
@@ -265,15 +273,12 @@ latex_elements = {
     # The paper size ('letterpaper' or 'a4paper').
     #
     # 'papersize': 'letterpaper',
-
     # The font size ('10pt', '11pt' or '12pt').
     #
     # 'pointsize': '10pt',
-
     # Additional stuff for the LaTeX preamble.
     #
     # 'preamble': '',
-
     # Latex figure (float) alignment
     #
     # 'figure_align': 'htbp',
@@ -283,20 +288,14 @@ latex_elements = {
 # (source start file, target name, title, author, documentclass
 # [howto, manual, or own class]).
 latex_documents = [
-    (master_doc, 'zfit.tex',
-     'zfit Documentation',
-     'zfit', 'manual'),
+    (master_doc, "zfit.tex", "zfit Documentation", "zfit", "manual"),
 ]
 
 # -- Options for manual page output ------------------------------------
 
 # One entry per manual page. List of tuples
 # (source start file, name, description, authors, manual section).
-man_pages = [
-    (master_doc, 'zfit',
-     'zfit Documentation',
-     [author], 1)
-]
+man_pages = [(master_doc, "zfit", "zfit Documentation", [author], 1)]
 
 # -- Options for Texinfo output ----------------------------------------
 
@@ -304,12 +303,15 @@ man_pages = [
 # (source start file, target name, title, author,
 #  dir menu entry, description, category)
 texinfo_documents = [
-    (master_doc, 'zfit',
-     'zfit Documentation',
-     author,
-     'zfit',
-     'One line description of project.',
-     'Miscellaneous'),
+    (
+        master_doc,
+        "zfit",
+        "zfit Documentation",
+        author,
+        "zfit",
+        "One line description of project.",
+        "Miscellaneous",
+    ),
 ]
 # cross reference
 default_role = "py:obj"
@@ -323,11 +325,11 @@ intersphinx_mapping = {
     # 'numdifftools': ('https://numdifftools.readthedocs.io/en/latest/index.html', None),
     "tensorflow": (
         "https://www.tensorflow.org/api_docs/python",
-        "https://raw.githubusercontent.com/GPflow/tensorflow-intersphinx/master/tf2_py_objects.inv"
+        "https://raw.githubusercontent.com/GPflow/tensorflow-intersphinx/master/tf2_py_objects.inv",
     ),
     "tensorflow_probability": (
         "https://www.tensorflow.org/probability/api_docs/python",
-        " https://raw.githubusercontent.com/GPflow/tensorflow-intersphinx/master/tfp_py_objects.inv"
+        " https://raw.githubusercontent.com/GPflow/tensorflow-intersphinx/master/tfp_py_objects.inv",
     ),
-    'uproot': ('https://uproot.readthedocs.io/en/latest/', None),
+    "uproot": ("https://uproot.readthedocs.io/en/latest/", None),
 }
