@@ -5,13 +5,15 @@ This may has some overhead in the beginning and won't be instantly fast compared
 Copyright (c) 2021 zfit
 """
 
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 import numpy as np
+import tensorflow as tf
 
 import zfit
 
-# set everything to numpy mode
+# set everything to numpy mode. This is only needed if we
+# don't use
 zfit.run.set_autograd_mode(False)
 zfit.run.set_graph_mode(False)
 
@@ -20,7 +22,7 @@ minimizer = zfit.minimize.IpyoptV1()
 
 
 # minimizer = zfit.minimize.Minuit()
-# minimizer = zfit.minimize.ScipyTrustKrylovV1()
+# minimizer = zfit.minimize.ScipyTrustConstrV1()
 # minimizer = zfit.minimize.NLoptLBFGSV1()
 
 
@@ -29,6 +31,14 @@ def func(x):
     return np.sum((x - 0.1) ** 2 + x[1] ** 4)
 
 
+# We can use the same in pure TF, then we can also use
+# the analytic gradient
+# @tf.function
+# def func(x):
+#     x = tf.convert_to_tensor(x)  # make sure it's an array
+#     return tf.reduce_sum((x - 0.1) ** 2 + x[1] ** 4)
+
+
 # we can also use a more complicated function instead
 # from scipy.optimize import rosen as func
 
