@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 import numpy as np
 
@@ -15,14 +15,14 @@ sigma = zfit.Parameter("sigma", 1.3, 0.5, 10)
 gauss = zfit.pdf.Gauss(mu=mu, sigma=sigma, obs=obs)
 
 # data
-normal_np = np.random.normal(loc=2., scale=3., size=10000)
+normal_np = np.random.normal(loc=2.0, scale=3.0, size=10000)
 data = zfit.Data.from_numpy(obs=obs, array=normal_np)
 
 # create NLL
 nll = zfit.loss.UnbinnedNLL(model=gauss, data=data)
 
-# # create a minimizer
-minimizer = zfit.minimize.Minuit()
+# create a minimizer
+minimizer = zfit.minimize.Minuit(gradient=False)
 result = minimizer.minimize(nll)
 
 # do the error calculations with a hessian approximation
@@ -30,3 +30,5 @@ param_errors = result.hesse()
 
 # or here with minos
 param_errors_asymetric, new_result = result.errors()
+
+print(result)
