@@ -1,17 +1,18 @@
-#  Copyright (c) 2020 zfit
+#  Copyright (c) 2022 zfit
 
 import pytest
 
 import zfit
 from zfit.core.dimension import limits_overlap, obs_subsets
-from zfit.core.space import combine_spaces
-from zfit.core.space import limits_consistent
-# noinspection PyUnresolvedReferences
-from zfit.core.testing import setup_function, teardown_function, tester
-from zfit.util.exception import (SpaceIncompatibleError,
-                                 ObsIncompatibleError, MultipleLimitsNotImplementedError, LimitsNotSpecifiedError)
+from zfit.core.space import combine_spaces, limits_consistent
+from zfit.util.exception import (
+    LimitsNotSpecifiedError,
+    MultipleLimitsNotImplemented,
+    ObsIncompatibleError,
+    SpaceIncompatibleError,
+)
 
-obs = ['obs' + str(i) for i in range(4)]
+obs = ["obs" + str(i) for i in range(4)]
 space1 = zfit.Space(obs=obs)
 space2 = zfit.Space(obs=reversed(obs))
 space3 = zfit.Space(obs=obs)
@@ -51,7 +52,7 @@ space2d_2 = zfit.Space(obs=obs[:2], limits=(lower2d_2, upper2d_2))
 
 def test_check_n_obs():
     with pytest.raises(SpaceIncompatibleError):
-        zfit.pdf.Gauss(1., 4., obs=space2d_1)
+        zfit.pdf.Gauss(1.0, 4.0, obs=space2d_1)
 
 
 def test_combine_spaces():
@@ -92,7 +93,7 @@ def test_add_spaces():
 
     with pytest.raises(ObsIncompatibleError):
         add_spaces(space1d_2, space2d_1)
-    with pytest.raises(MultipleLimitsNotImplementedError):
+    with pytest.raises(MultipleLimitsNotImplemented):
         add_spaces(space1d_2, space1d_1).limits
 
 
@@ -114,31 +115,29 @@ def test_limits_overlap():
     assert limits_overlap([space1d_1, space1d_2])
     assert limits_overlap([space1d_1, space1d_2, space1d_12])
     assert limits_overlap([space1d_1, space2d_2, space1d_12])
-    assert not limits_overlap([space1d_1, space2d_2, space1d_12], allow_exact_match=True)
+    assert not limits_overlap(
+        [space1d_1, space2d_2, space1d_12], allow_exact_match=True
+    )
 
 
 def test_obs_subsets():
-    space1 = zfit.Space('obs1')
-    space2 = zfit.Space(['obs1', 'obs2', 'obs4'])
-    space5 = zfit.Space(['obs3'])
-    space6 = zfit.Space('obs6')
-    space4 = zfit.Space(['obs3', 'obs7'])
-    space3 = zfit.Space(['obs4', 'obs5'])
-    spaces = [space1,
-              space2,
-              space5,
-              space6,
-              space4,
-              space3]
+    space1 = zfit.Space("obs1")
+    space2 = zfit.Space(["obs1", "obs2", "obs4"])
+    space5 = zfit.Space(["obs3"])
+    space6 = zfit.Space("obs6")
+    space4 = zfit.Space(["obs3", "obs7"])
+    space3 = zfit.Space(["obs4", "obs5"])
+    spaces = [space1, space2, space5, space6, space4, space3]
     obs_subset = obs_subsets(spaces)
 
-    obs1comb = frozenset(['obs1', 'obs2', 'obs4', 'obs5'])
-    obs37comb = frozenset(['obs3', 'obs7'])
-    obs6 = frozenset(['obs6'])
-    true_obs = {obs1comb,
-                obs37comb,
-                obs6,
-                }
+    obs1comb = frozenset(["obs1", "obs2", "obs4", "obs5"])
+    obs37comb = frozenset(["obs3", "obs7"])
+    obs6 = frozenset(["obs6"])
+    true_obs = {
+        obs1comb,
+        obs37comb,
+        obs6,
+    }
 
     assert true_obs == set(obs_subset.keys())
     assert obs_subset[obs1comb] == [space1, space2, space3]
