@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 import numpy as np
 import pytest
 import scipy
@@ -9,18 +9,22 @@ import zfit
 
 
 def func_cosxy2_np(x, y):
-    return np.cos(x * y ** 2)
+    return np.cos(x * y**2)
 
 
 def func_cosxy2_tf(x, y):
-    return tf.cos(x * y ** 2)
+    return tf.cos(x * y**2)
 
 
 def integral_y(x, lowery, uppery):
     def indef_limit(limit):
-        integral = (np.sqrt(2) * np.sqrt(np.pi) * scipy.special.fresnel(
-            np.sqrt(2) * limit * np.sqrt(x) / np.sqrt(np.pi))[1] * scipy.special.gamma(1 / 4) / (
-                        8 * np.sqrt(x) * scipy.special.gamma(5 / 4)))
+        integral = (
+            np.sqrt(2)
+            * np.sqrt(np.pi)
+            * scipy.special.fresnel(np.sqrt(2) * limit * np.sqrt(x) / np.sqrt(np.pi))[1]
+            * scipy.special.gamma(1 / 4)
+            / (8 * np.sqrt(x) * scipy.special.gamma(5 / 4))
+        )
         return integral
 
     return indef_limit(uppery) - indef_limit(lowery)
@@ -28,7 +32,7 @@ def integral_y(x, lowery, uppery):
 
 def integral_x(y, lowerx, upperx):
     def indef_limit(limit):
-        return np.sin(limit * y ** 2) / y ** 2
+        return np.sin(limit * y**2) / y**2
 
     return indef_limit(upperx) - indef_limit(lowerx)
 
@@ -49,17 +53,17 @@ def test_partial_integral():
 
     data_np = np.linspace((1, 1), (4, 4), 4000)
     data = zfit.Data.from_numpy(obs=obs, array=data_np)
-    probs = cosxy2.pdf(data)
+    probs = cosxy2.pdf(data, norm=False)
     probs_np = probs.numpy()
     x = data_np[:, 0]
     y = data_np[:, 1]
     probs_func = func_cosxy2_np(x=x, y=y)
     ratios = probs_np / probs_func
-    assert pytest.approx(0., rel=0.0001) == np.std(ratios)  # ratio should be constant
+    assert pytest.approx(0.0, rel=0.0001) == np.std(ratios)  # ratio should be constant
     ratio = np.average(ratios)
 
-    integral_x_tf = cosxy2.partial_integrate(x=data, limits=xspace)
-    integral_y_tf = cosxy2.partial_integrate(x=data, limits=yspace)
+    integral_x_tf = cosxy2.partial_integrate(x=data, limits=xspace, norm=False)
+    integral_y_tf = cosxy2.partial_integrate(x=data, limits=yspace, norm=False)
     integral_x_np = integral_x_tf.numpy()
     integral_y_np = integral_y_tf.numpy()
 
