@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 import numpy as np
 import pytest
 from scipy.stats import crystalball
@@ -22,10 +22,10 @@ rbounds = (mu, bounds[1])
 
 
 def _cb_params_factory(name_add=""):
-    mu_ = zfit.Parameter('mu_cb' + name_add, mu)
-    sigma_ = zfit.Parameter('sigma_cb' + name_add, sigma)
-    alphal_ = zfit.Parameter('alphal_cb' + name_add, alphal)
-    nl_ = zfit.Parameter('nl_cb' + name_add, nl)
+    mu_ = zfit.Parameter(f"mu_cb{name_add}", mu)
+    sigma_ = zfit.Parameter(f"sigma_cb{name_add}", sigma)
+    alphal_ = zfit.Parameter(f"alphal_cb{name_add}", alphal)
+    nl_ = zfit.Parameter(f"nl_cb{name_add}", nl)
     return {"mu": mu_, "sigma": sigma_, "alpha": alphal_, "n": nl_}
 
 
@@ -48,37 +48,49 @@ def eval_testing(pdf, x):
 
 
 def test_cb_integral():
-    obs = zfit.Space('x', limits=bounds)
+    obs = zfit.Space("x", limits=bounds)
 
-    mu_ = zfit.Parameter('mu_cb5', mu)
-    sigma_ = zfit.Parameter('sigma_cb5', sigma)
-    alphal_ = zfit.Parameter('alphal_cb5', alphal)
-    nl_ = zfit.Parameter('nl_cb5', nl)
+    mu_ = zfit.Parameter("mu_cb5", mu)
+    sigma_ = zfit.Parameter("sigma_cb5", sigma)
+    alphal_ = zfit.Parameter("alphal_cb5", alphal)
+    nl_ = zfit.Parameter("nl_cb5", nl)
 
     cbl = CrystalBall(obs=obs, mu=mu_, sigma=sigma_, alpha=alphal_, n=nl_)
     int_limits = (-1, 3)
-    integral_numeric = cbl.numeric_integrate(limits=int_limits, norm_range=False)
+    integral_numeric = cbl.numeric_integrate(limits=int_limits, norm=False)
 
-    integral = cbl.analytic_integrate(limits=int_limits, norm_range=False)
+    integral = cbl.analytic_integrate(limits=int_limits, norm=False)
     integral_numeric = zfit.run(integral_numeric)
     integral = zfit.run(integral)
 
-    assert pytest.approx(integral_numeric, integral, 1e-5)
+    assert pytest.approx(integral_numeric, 1e-5) == integral
+
+    rnd_limits = sorted(list(np.random.uniform(*bounds, 13)) + list(bounds))
+    integrals = [
+        cbl.integrate((low, up), norm=False)
+        for low, up in zip(rnd_limits[:-1], rnd_limits[1:])
+    ]
+
+    integral = np.sum(integrals)
+    integral_full = zfit.run(cbl.integrate(bounds, norm=False))
+    assert pytest.approx(float(integral_full)) == float(integral)
 
 
 def test_cb_dcb():
-    obs = zfit.Space('x', limits=bounds)
+    obs = zfit.Space("x", limits=bounds)
 
-    mu_ = zfit.Parameter('mu_cb5', mu)
-    sigma_ = zfit.Parameter('sigma_cb5', sigma)
-    alphal_ = zfit.Parameter('alphal_cb5', alphal)
-    nl_ = zfit.Parameter('nl_cb5', nl)
-    alphar_ = zfit.Parameter('alphar_cb5', alphar)
-    nr_ = zfit.Parameter('nr_cb5', nr)
+    mu_ = zfit.Parameter("mu_cb5", mu)
+    sigma_ = zfit.Parameter("sigma_cb5", sigma)
+    alphal_ = zfit.Parameter("alphal_cb5", alphal)
+    nl_ = zfit.Parameter("nl_cb5", nl)
+    alphar_ = zfit.Parameter("alphar_cb5", alphar)
+    nr_ = zfit.Parameter("nr_cb5", nr)
 
     cbl = CrystalBall(obs=obs, mu=mu_, sigma=sigma_, alpha=alphal_, n=nl_)
     cbr = CrystalBall(obs=obs, mu=mu_, sigma=sigma_, alpha=-alphar_, n=nr_)
-    dcb = DoubleCB(obs=obs, mu=mu_, sigma=sigma_, alphal=alphal_, nl=nl_, alphar=alphar_, nr=nr_)
+    dcb = DoubleCB(
+        obs=obs, mu=mu_, sigma=sigma_, alphal=alphal_, nl=nl_, alphar=alphar_, nr=nr_
+    )
 
     sample_testing(cbl)
     sample_testing(cbr)
@@ -93,7 +105,7 @@ def test_cb_dcb():
     assert not any(np.isnan(probsr))
 
     probsl_scipy = crystalball.pdf(x, beta=alphal, m=nl, loc=mu, scale=sigma)
-    probsr_scipy = crystalball.pdf(-x + 2 * mu, beta=alphar, m=nr, loc=mu, scale=sigma)  # mirrored PDF
+    probsr_scipy = crystalball.pdf(-x + 2 * mu, beta=alphar, m=nr, loc=mu, scale=sigma)
 
     # We take the ration as the normalization is not fixed
     ratio_l = probsl_scipy / probsl
@@ -104,7 +116,7 @@ def test_cb_dcb():
 
     kwargs = dict(limits=(-5.0, mu), norm_range=lbounds)
     intl = cbl.integrate(**kwargs) - dcb.integrate(**kwargs)
-    assert pytest.approx(intl.numpy(), abs=1e-3) == 0.
+    assert pytest.approx(intl.numpy(), abs=1e-3) == 0.0
     intl = cbr.integrate(**kwargs) - dcb.integrate(**kwargs)
     assert pytest.approx(intl.numpy(), abs=1e-3) != 0
 
@@ -112,9 +124,9 @@ def test_cb_dcb():
     kwargs = dict(limits=(mu, 2.0), norm_range=rbounds)
     dcb_integr1 = dcb.integrate(**kwargs)
     intr = cbr.integrate(**kwargs) - dcb_integr1
-    assert pytest.approx(intr.numpy(), abs=1e-3) == 0.
+    assert pytest.approx(intr.numpy(), abs=1e-3) == 0.0
     intr = cbl.integrate(**kwargs) - dcb.integrate(**kwargs)
-    assert pytest.approx(intr.numpy(), abs=1e-3) != 0.
+    assert pytest.approx(intr.numpy(), abs=1e-3) != 0.0
 
     xl = x[x <= mu]
     xr = x[x > mu]
@@ -130,3 +142,12 @@ def test_cb_dcb():
 
     assert np.allclose(ratio_l, ratio_l[0])
     assert np.allclose(ratio_r, ratio_r[0])
+
+    rnd_limits = sorted(list(np.random.uniform(*bounds, 130)) + list(bounds))
+    integrals = []
+    for low, up in zip(rnd_limits[:-1], rnd_limits[1:]):
+        integrals.append(dcb.integrate((low, up), norm=False))
+
+    integral = np.sum(integrals)
+    integral_full = zfit.run(dcb.integrate((bounds[0], up), norm=False))
+    assert pytest.approx(float(integral_full)) == float(integral)
