@@ -1,11 +1,9 @@
-#  Copyright (c) 2020 zfit
+#  Copyright (c) 2022 zfit
 import numpy as np
 import pytest
 
 import zfit
 from zfit import Parameter
-# noinspection PyUnresolvedReferences
-from zfit.core.testing import setup_function, teardown_function, tester
 from zfit.models.dist_tfp import Poisson
 
 lamb_true = 50
@@ -21,8 +19,21 @@ def create_poisson():
     return poisson
 
 
-def test_poisson():
-    poisson = create_poisson()
+def create_poisson_composed_rate():
+    N1 = Parameter("N1", lamb_true / 2)
+    N2 = Parameter("N2", lamb_true / 2)
+    N = zfit.param.ComposedParameter("N", lambda n1, n2: n1 + n2, params=[N1, N2])
+
+    poisson = Poisson(obs=obs, lamb=N)
+    return poisson
+
+
+@pytest.mark.parametrize("composed_rate", [False, True])
+def test_poisson(composed_rate):
+    if composed_rate:
+        poisson = create_poisson_composed_rate()
+    else:
+        poisson = create_poisson()
 
     probs1 = poisson.pdf(x=test_values)
     probs1 = probs1.numpy()
