@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 
 # deactivating CUDA capable gpus
@@ -22,9 +22,11 @@ prec = 0.00001
 
 def test_polynomial():
     coeffs = [5.3, 1.2, complex(1.3, 0.4), -42, 32.4, 529.3, -0.93]
-    x = tf.constant(5.)
-    polynom_tf = zfit.z.math.poly_complex(*(coeffs + [x]))  # py34 comp: *x, y does not work
-    polynom_np = np.polyval(coeffs[::-1], 5.)
+    x = tf.constant(5.0)
+    polynom_tf = zfit.z.math.poly_complex(
+        *(coeffs + [x])
+    )  # py34 comp: *x, y does not work
+    polynom_np = np.polyval(coeffs[::-1], 5.0)
 
     result = polynom_tf.numpy()
     assert result == pytest.approx(polynom_np, rel=prec)
@@ -41,7 +43,7 @@ def test_auto_upcast():
     assert tensor_from_i32.dtype == tf.int64
     assert tensor_from_i64.dtype == tf.int64
 
-    tensor_from_c64 = _auto_upcast(tf.constant(5., dtype=tf.complex64))
-    tensor_from_c128 = _auto_upcast(tf.constant(5., dtype=tf.complex128))
+    tensor_from_c64 = _auto_upcast(tf.constant(5.0, dtype=tf.complex64))
+    tensor_from_c128 = _auto_upcast(tf.constant(5.0, dtype=tf.complex128))
     assert tensor_from_c64.dtype == tf.complex128
     assert tensor_from_c128.dtype == tf.complex128
