@@ -1,8 +1,10 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
 
 import abc
 from collections import OrderedDict
-from typing import Callable, Dict, Optional, Union
+from collections.abc import Callable
 
 import tensorflow as tf
 import tensorflow_probability as tfp
@@ -10,24 +12,25 @@ from ordered_set import OrderedSet
 
 import zfit.z.numpy as znp
 from zfit import z
-
+from .baseobject import BaseNumeric
+from .dependents import _extract_dependencies
+from .interfaces import ZfitConstraint, ZfitParameter
 from ..settings import ztypes
 from ..util import ztyping
 from ..util.container import convert_to_container
 from ..util.exception import ShapeIncompatibleError
-from .baseobject import BaseNumeric
-from .dependents import _extract_dependencies
-from .interfaces import ZfitConstraint, ZfitParameter
-from .parameter import convert_to_parameter
 
 tfd = tfp.distributions
 
 
 class BaseConstraint(ZfitConstraint, BaseNumeric):
-
-    def __init__(self, params: Union[Dict[str, ZfitParameter]] = None,
-                 name: str = "BaseConstraint", dtype=ztypes.float,
-                 **kwargs):
+    def __init__(
+        self,
+        params: dict[str, ZfitParameter] = None,
+        name: str = "BaseConstraint",
+        dtype=ztypes.float,
+        **kwargs,
+    ):
         """Base class for constraints.
 
         Args:
@@ -50,19 +53,20 @@ class BaseConstraint(ZfitConstraint, BaseNumeric):
 
 
 class SimpleConstraint(BaseConstraint):
-
-    def __init__(self, func: Callable, params: Optional[ztyping.ParameterType]):
+    def __init__(self, func: Callable, params: ztyping.ParameterType | None):
         """Constraint from a (function returning a) Tensor.
 
         The parameters are named "param_{i}" with i starting from 0 and corresponding to the index of params.
 
         Args:
             func: Callable that constructs the constraint and returns a tensor.
-            params: The dependents (independent `zfit.Parameter`) of the loss. If not given, the
+            params: The dependents (independent ``zfit.Parameter``) of the loss. If not given, the
                 dependents are figured out automatically.
         """
         self._simple_func = func
-        self._simple_func_dependents = convert_to_container(params, container=OrderedSet)
+        self._simple_func_dependents = convert_to_container(
+            params, container=OrderedSet
+        )
 
         params = convert_to_container(params, container=list)
         params = OrderedDict((f"param_{i}", p) for i, p in enumerate(params))
@@ -74,10 +78,14 @@ class SimpleConstraint(BaseConstraint):
 
 
 class ProbabilityConstraint(BaseConstraint):
-
-    def __init__(self, observation: Union[ztyping.NumericalScalarType, ZfitParameter],
-                 params: Union[Dict[str, ZfitParameter]] = None, name: str = "ProbabilityConstraint",
-                 dtype=ztypes.float, **kwargs):
+    def __init__(
+        self,
+        observation: ztyping.NumericalScalarType | ZfitParameter,
+        params: dict[str, ZfitParameter] = None,
+        name: str = "ProbabilityConstraint",
+        dtype=ztypes.float,
+        **kwargs,
+    ):
         """Base class for constraints using a probability density function.
 
         Args:
@@ -87,20 +95,24 @@ class ProbabilityConstraint(BaseConstraint):
             observation: Observed values of the parameter
                 to constraint obtained from auxiliary measurements.
         """
-
+        # TODO: proper handling of input params, arrays. ArrayParam?
+        params = convert_to_container(params)
         params_dict = {f"param_{i}": p for i, p in enumerate(params)}
         super().__init__(name=name, dtype=dtype, params=params_dict, **kwargs)
+        params = tuple(self.params.values())
 
         observation = convert_to_container(observation, tuple)
         if len(observation) != len(params):
-            raise ShapeIncompatibleError("observation and params have to be the same length. Currently"
-                                         f"observation: {len(observation)}, params: {len(params)}")
+            raise ShapeIncompatibleError(
+                "observation and params have to be the same length. Currently"
+                f"observation: {len(observation)}, params: {len(params)}"
+            )
 
-        self._observation = []
-        for obs, p in zip(observation, params):
-            obs = convert_to_parameter(obs, f"{p.name}_obs", prefer_constant=False)
-            obs.floating = False
-            self._observation.append(obs)
+        self._observation = observation  # TODO: needed below? Why?
+        # for obs, p in zip(observation, params):
+        #     obs = convert_to_parameter(obs, f"{p.name}_obs", prefer_constant=False)
+        #     obs.floating = False
+        #     self._observation.append(obs)
 
         self._ordered_params = params
 
@@ -120,14 +132,14 @@ class ProbabilityConstraint(BaseConstraint):
         return _extract_dependencies(self.get_params())
 
     def sample(self, n):
-        """Sample `n` points from the probability density function for the observed value of the parameters.
+        """Sample ``n`` points from the probability density function for the observed value of the parameters.
 
         Args:
             n: The number of samples to be generated.
         Returns:
         """
         sample = self._sample(n=n)
-        return {p: sample[:, i] for i, p in enumerate(self.observation)}
+        return {p: sample[:, i] for i, p in enumerate(self._ordered_params)}
 
     @abc.abstractmethod
     def _sample(self, n):
@@ -139,18 +151,26 @@ class ProbabilityConstraint(BaseConstraint):
 
 
 class TFProbabilityConstraint(ProbabilityConstraint):
-
-    def __init__(self, observation: Union[ztyping.NumericalScalarType, ZfitParameter],
-                 params: Dict[str, ZfitParameter], distribution: tfd.Distribution,
-                 dist_params, dist_kwargs=None, name: str = "DistributionConstraint", dtype=ztypes.float,
-                 **kwargs):
-        """Base class for constraints using a probability density function from `tensorflow_probability`.
+    def __init__(
+        self,
+        observation: ztyping.NumericalScalarType | ZfitParameter,
+        params: dict[str, ZfitParameter],
+        distribution: tfd.Distribution,
+        dist_params,
+        dist_kwargs=None,
+        name: str = "DistributionConstraint",
+        dtype=ztypes.float,
+        **kwargs,
+    ):
+        """Base class for constraints using a probability density function from ``tensorflow_probability``.
 
         Args:
             distribution: The probability density function
                 used to constraint the parameters
         """
-        super().__init__(observation=observation, params=params, name=name, dtype=dtype, **kwargs)
+        super().__init__(
+            observation=observation, params=params, name=name, dtype=dtype, **kwargs
+        )
 
         self._distribution = distribution
         self.dist_params = dist_params
@@ -168,20 +188,22 @@ class TFProbabilityConstraint(ProbabilityConstraint):
 
     def _value(self):
         value = -self.distribution.log_prob(self._params_array)
-        return value
+        return tf.reduce_sum(value)
 
     def _sample(self, n):
-        # TODO cache: add proper caching
         return self.distribution.sample(n)
 
 
 class GaussianConstraint(TFProbabilityConstraint):
-
-    def __init__(self, params: ztyping.ParamTypeInput, observation: ztyping.NumericalScalarType,
-                 uncertainty: ztyping.NumericalScalarType):
+    def __init__(
+        self,
+        params: ztyping.ParamTypeInput,
+        observation: ztyping.NumericalScalarType,
+        uncertainty: ztyping.NumericalScalarType,
+    ):
         r"""Gaussian constraints on a list of parameters to some observed values with uncertainties.
 
-        A Gaussian constraint is defined as the likelihood of `params` given the `observations` and `uncertainty` from
+        A Gaussian constraint is defined as the likelihood of ``params`` given the ``observations`` and ``uncertainty`` from
         a different measurement.
 
         .. math::
@@ -211,26 +233,40 @@ class GaussianConstraint(TFProbabilityConstraint):
             if sigma.shape.ndims > 1:
                 covariance = sigma  # TODO: square as well?
             elif sigma.shape.ndims == 1:
-                covariance = tf.linalg.tensor_diag(z.pow(sigma, 2.))
+                covariance = tf.linalg.tensor_diag(z.pow(sigma, 2.0))
             else:
                 sigma = znp.reshape(sigma, [1])
-                covariance = tf.linalg.tensor_diag(z.pow(sigma, 2.))
+                covariance = tf.linalg.tensor_diag(z.pow(sigma, 2.0))
 
-            if not params_tensor.shape[0] == mu.shape[0] == covariance.shape[0] == covariance.shape[1]:
-                raise ShapeIncompatibleError(f"params_tensor, observation and uncertainty have to have the"
-                                             " same length. Currently"
-                                             f"param: {params_tensor.shape[0]}, mu: {mu.shape[0]}, "
-                                             f"covariance (from uncertainty): {covariance.shape[0:2]}")
+            if (
+                not params_tensor.shape[0]
+                == mu.shape[0]
+                == covariance.shape[0]
+                == covariance.shape[1]
+            ):
+                raise ShapeIncompatibleError(
+                    f"params_tensor, observation and uncertainty have to have the"
+                    " same length. Currently"
+                    f"param: {params_tensor.shape[0]}, mu: {mu.shape[0]}, "
+                    f"covariance (from uncertainty): {covariance.shape[0:2]}"
+                )
             return covariance
 
         distribution = tfd.MultivariateNormalTriL
-        dist_params = lambda observation: dict(loc=observation,
-                                               scale_tril=tf.linalg.cholesky(
-                                                   create_covariance(observation, uncertainty)))
+        dist_params = lambda observation: dict(
+            loc=observation,
+            scale_tril=tf.linalg.cholesky(create_covariance(observation, uncertainty)),
+        )
         dist_kwargs = dict(validate_args=True)
 
-        super().__init__(name="GaussianConstraint", observation=observation, params=params,
-                         distribution=distribution, dist_params=dist_params, dist_kwargs=dist_kwargs)
+        super().__init__(
+            name="GaussianConstraint",
+            observation=observation,
+            params=params,
+            distribution=distribution,
+            dist_params=dist_params,
+            dist_kwargs=dist_kwargs,
+        )
 
         self._covariance = lambda: create_covariance(self.observation, uncertainty)
 
@@ -238,3 +274,90 @@ class GaussianConstraint(TFProbabilityConstraint):
     def covariance(self):
         """Return the covariance matrix of the observed values of the parameters constrained."""
         return self._covariance()
+
+
+class PoissonConstraint(TFProbabilityConstraint):
+    def __init__(
+        self, params: ztyping.ParamTypeInput, observation: ztyping.NumericalScalarType
+    ):
+        r"""Poisson constraints on a list of parameters to some observed values.
+
+        Constraints parameters that can be counts (i.e. from a histogram) or, more generally, are
+        Poisson distributed. This is often used in the case of histogram templates which are obtained
+        from simulation and have a poisson uncertainty due to limited statistics.
+
+        .. math::
+            \text{constraint} = \text{Poisson}(\text{observation}; \text{params})
+
+
+        Args:
+            params: The parameters to constraint; corresponds to the mu in the Poisson
+                distribution.
+            observation: observed values of the parameter; corresponds to lambda
+                in the Poisson distribution.
+        Raises:
+            ShapeIncompatibleError: If params and observation have incompatible shapes.
+        """
+
+        observation = convert_to_container(observation, tuple)
+        # observation = tuple(convert_to_parameter(obs) for obs in observation)
+        params = convert_to_container(params, tuple)
+
+        distribution = tfd.Poisson
+        dist_params = dict(rate=observation)
+        dist_kwargs = dict(validate_args=False)
+
+        super().__init__(
+            name="PoissonConstraint",
+            observation=observation,
+            params=params,
+            distribution=distribution,
+            dist_params=dist_params,
+            dist_kwargs=dist_kwargs,
+        )
+
+
+class LogNormalConstraint(TFProbabilityConstraint):
+    def __init__(
+        self,
+        params: ztyping.ParamTypeInput,
+        observation: ztyping.NumericalScalarType,
+        uncertainty: ztyping.NumericalScalarType,
+    ):
+        r"""Log-normal constraints on a list of parameters to some observed values.
+
+        Constraints parameters that can be counts (i.e. from a histogram) or, more generally, are
+        LogNormal distributed. This is often used in the case of histogram templates which are obtained
+        from simulation and have a log-normal uncertainty due to a multiplicative uncertainty.
+
+        .. math::
+            \text{constraint} = \text{LogNormal}(\text{observation}; \text{params})
+
+
+        Args:
+            params: The parameters to constraint; corresponds to the mu in the Poisson
+                distribution.
+            observation: observed values of the parameter; corresponds to lambda
+                in the Poisson distribution.
+            uncertainty: uncertainty of the observed values of the parameter; corresponds to sigma
+                in the Poisson distribution.
+        Raises:
+            ShapeIncompatibleError: If params, mu and sigma have incompatible shapes.
+        """
+
+        observation = convert_to_container(observation, tuple)
+        params = convert_to_container(params, tuple)
+        uncertainty = convert_to_container(uncertainty, tuple)
+
+        distribution = tfd.LogNormal
+        dist_params = lambda observation: dict(loc=observation, scale=uncertainty)
+        dist_kwargs = dict(validate_args=False)
+
+        super().__init__(
+            name="LogNormalConstraint",
+            observation=observation,
+            params=params,
+            distribution=distribution,
+            dist_params=dist_params,
+            dist_kwargs=dist_kwargs,
+        )
