@@ -1,52 +1,77 @@
-#  Copyright (c) 2021 zfit
-import warnings
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Union
+
+import xxhash
+from tensorflow.python.util.deprecation import deprecated_args, deprecated
+
+if TYPE_CHECKING:
+    import zfit
+
+from collections.abc import Mapping
+from collections.abc import Callable
+
 from collections import OrderedDict
 from contextlib import ExitStack
-from typing import Callable, Dict, List, Mapping, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import tensorflow as tf
 import uproot
-from tensorflow.python.ops import array_ops
 
 # from ..settings import types as ztypes
 import zfit
 import zfit.z.numpy as znp
 
 from .. import z
-from ..settings import ztypes
+from ..settings import ztypes, run
 from ..util import ztyping
 from ..util.cache import GraphCachable, invalidate_graph
 from ..util.container import convert_to_container
-from ..util.exception import (LogicalUndefinedOperationError,
-                              ObsIncompatibleError, ShapeIncompatibleError,
-                              WorkInProgressError)
+from ..util.exception import (
+    ObsIncompatibleError,
+    ShapeIncompatibleError,
+    WorkInProgressError,
+)
 from ..util.temporary import TemporarilySet
 from .baseobject import BaseObject
 from .coordinates import convert_to_obs_str
 from .dimension import BaseDimensional
-from .interfaces import ZfitData, ZfitSpace
-from .parameter import register_tensor_conversion
+from .interfaces import ZfitSpace, ZfitUnbinnedData
+from .tensorlike import register_tensor_conversion, OverloadableMixin
 from .space import Space, convert_to_space
 
 
 # TODO: make cut only once, then remember
-class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
+class Data(
+    ZfitUnbinnedData, BaseDimensional, BaseObject, OverloadableMixin, GraphCachable
+):
     BATCH_SIZE = 1000000  # 1 mio
 
-    def __init__(self, dataset: Union[tf.data.Dataset, "LightDataset"], obs: ztyping.ObsTypeInput = None,
-                 name: str = None, weights=None, iterator_feed_dict: Dict = None,
-                 dtype: tf.DType = None):
-        """Create a data holder from a `dataset` used to feed into `models`.
+    def __init__(
+        self,
+        dataset: tf.data.Dataset | LightDataset,
+        obs: ztyping.ObsTypeInput = None,
+        name: str = None,
+        weights=None,
+        dtype: tf.DType = None,
+        use_hash: bool = None,
+    ):
+        """Create a data holder from a ``dataset`` used to feed into ``models``.
 
         Args:
             dataset: A dataset storing the actual values
             obs: Observables where the data is defined in
-            name: Name of the `Data`
-            iterator_feed_dict:
+            name: Name of the ``Data``
+            weights: Weights of the data
             dtype: |dtype_arg_descr|
+            use_hash: Whether to use a hash for caching
         """
+        if use_hash is None:
+            use_hash = run.hashing_data()
+        self._use_hash = use_hash
         if name is None:
             name = "Data"
         if dtype is None:
@@ -63,13 +88,15 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         self._data_range = None
         self._set_space(obs)
         self._original_space = self.space
-        self._data_range = self.space  # TODO proper data cuts: currently set so that the cuts in all dims are applied
-        self.batch_size = self.BATCH_SIZE
-        self.dataset = dataset.batch(self.batch_size)
+        self._data_range = (
+            self.space
+        )  # TODO proper data cuts: currently set so that the cuts in all dims are applied
+        self.dataset = dataset.batch(100_000_000)
         self._name = name
-        self.iterator_feed_dict = iterator_feed_dict
-        self.iterator = None
+
         self.set_weights(weights=weights)
+        self._hashint = None
+        self._update_hash()
 
     @property
     def nevents(self):
@@ -78,6 +105,10 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
             nevents = self._get_nevents()
         return nevents
 
+    @property
+    def hashint(self) -> int | None:
+        return self._hashint
+
     # TODO: which naming? nevents or n_events
 
     @property
@@ -101,6 +132,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         self._check_n_obs(space=obs)
         obs = obs.with_autofill_axes(overwrite=True)
         self._space = obs
+        self._update_hash()
 
     @property
     def data_range(self):
@@ -115,6 +147,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
 
         def setter(value):
             self._data_range = value
+            self._update_hash()
 
         def getter():
             return self._data_range
@@ -132,6 +165,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
             weights = self._weights
         return weights
 
+    @deprecated(None, "Do not set the weights on a data set, create a new one instead.")
     @invalidate_graph
     def set_weights(self, weights: ztyping.WeightsInputType):
         """Set (temporarily) the weights of the dataset.
@@ -147,6 +181,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
 
         def setter(value):
             self._weights = value
+            self._update_hash()
 
         def getter():
             return self.weights
@@ -154,59 +189,126 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         return TemporarilySet(value=weights, getter=getter, setter=setter)
 
     @property
-    def space(self) -> "ZfitSpace":
+    def space(self) -> ZfitSpace:
         return self._space
 
-    # constructors
     @classmethod
-    def from_root_iter(cls, path, treepath, branches=None, entrysteps=None, name=None, **kwargs):
-        # branches = convert_to_container(branches)
-        raise RuntimeWarning("Currently, this is not supported.")
+    def from_pandas(
+        cls,
+        df: pd.DataFrame,
+        obs: ztyping.ObsTypeInput = None,
+        weights: ztyping.WeightsInputType | str = None,
+        name: str = None,
+        dtype: tf.DType = None,
+        use_hash: bool = None,
+    ):
+        """Create a ``Data`` from a pandas DataFrame. If ``obs`` is ``None``, columns are used as obs.
+
+        Args:
+            df: pandas DataFrame that contains the data. If ``obs`` is ``None``, columns are used as obs. Can be
+                a superset of obs.
+            obs: obs to use for the data. obs have to be the columns in the data frame.
+                If ``None``, columns are used as obs.
+            weights: Weights of the data. Has to be 1-D and match the shape
+                of the data (nevents) or a string that is a column in the dataframe.
+            name:
+            dtype: dtype of the data
+            use_hash: If ``True``, a hash of the data is created and is used to identify it in caching.
+        """
+        if obs is None:
+            obs = list(df.columns)
+
+        obs = convert_to_space(obs)
+        not_in_df = set(obs.obs) - set(df.columns)
+        if not_in_df:
+            raise ValueError(
+                f"Observables {not_in_df} not in dataframe with columns {df.columns}"
+            )
+        if isinstance(weights, str):
+            if weights not in df.columns:
+                raise ValueError(
+                    f"Weights {weights} is a string and not in dataframe with columns {df.columns}"
+                )
+            weights = df[weights]
+        array = df[list(obs.obs)].values
+
+        return Data.from_numpy(  # *not* class, if subclass, keep constructor
+            obs=obs,
+            array=array,
+            weights=weights,
+            name=name,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
 
     @classmethod
-    def from_root(cls, path: str, treepath: str, branches: List[str] = None, branches_alias: Dict = None,
-                  weights: ztyping.WeightsStrInputType = None,
-                  name: str = None,
-                  dtype: tf.DType = None,
-                  root_dir_options=None) -> "Data":
-        """Create a `Data` from a ROOT file. Arguments are passed to `uproot`.
+    @deprecated_args(None, "Use obs instead.", "branches")
+    @deprecated_args(
+        None,
+        "Use obs_alias instead and make sure to invert the logic! I.e. it's a mapping from"
+        " the observable name to the actual branch name.",
+        "branches_alias",
+    )
+    def from_root(
+        cls,
+        path: str,
+        treepath: str,
+        obs: ZfitSpace = None,
+        *,
+        weights: ztyping.WeightsStrInputType = None,
+        obs_alias: Mapping[str, str] = None,
+        name: str = None,
+        dtype: tf.DType = None,
+        root_dir_options=None,
+        use_hash: bool = None,
+        # deprecated
+        branches: list[str] = None,
+        branches_alias: dict = None,
+    ) -> Data:
+        """Create a ``Data`` from a ROOT file. Arguments are passed to ``uproot``.
 
         The arguments are passed to uproot directly.
 
         Args:
-            path:
-            treepath:
-            branches:
-            branches_alias: A mapping from the `branches` (as keys) to the actual `observables` (as values).
-                This allows to have different `observable` names, independent of the branch name in the file.
+            path: Path to the root file.
+            treepath: Name of the tree in the root file.
+            obs: Observables of the data. This will also be the columns of the data if not *obs_alias* is given.
             weights: Weights of the data. Has to be 1-D and match the shape
                 of the data (nevents). Can be a column of the ROOT file by using a string corresponding to a
                 column.
+            obs_alias: A mapping from the ``obs`` (as keys) to the actual ``branches`` (as values) in the root file.
+                This allows to have different ``observable`` names, independent of the branch name in the file.
             name:
             root_dir_options:
 
         Returns:
-            `zfit.Data`:
+            ``zfit.Data``: A ``Data`` object containing the unbinned data.
         """
-        # TODO 0.6: use obs here instead of branches
-        # if branches:
-        #     warnings.warn(FutureWarning("`branches` is deprecated, please use `obs` instead"), stacklevel=2)
-        #     obs = branches
-        # obs = convert_to_space(obs)
-        # branches = obs.obs
-        if branches_alias is None and branches is None:
-            raise ValueError("Either branches or branches_alias has to be specified.")
+        # begin deprecated legacy arguments
+        if branches:
+            obs = branches
+            del branches
+        if branches_alias is not None:
+            if obs_alias is not None:
+                raise ValueError("Cannot use both `branches_alias` and `obs_alias`.")
+            obs_alias = {obs: branch for branch, obs in branches_alias.items()}
+            del branches_alias
 
-        if branches_alias is None:
-            branches_alias = {}
-        if branches is None:
-            branches = list(branches_alias.values())
-
-        weights_are_branch = isinstance(weights, str)
-
-        branches = convert_to_container(branches)
+        # end legacy
         if root_dir_options is None:
             root_dir_options = {}
+        if obs_alias is None and obs is None:
+            raise ValueError("Either branches or branches_alias has to be specified.")
+        if obs_alias is None:
+            obs_alias = {}
+        if obs is None:
+            obs = list(obs_alias.values())
+
+        obs = convert_to_space(obs)
+
+        branches = [obs_alias.get(branch, branch) for branch in obs.obs]
+
+        weights_are_branch = isinstance(weights, str)
 
         def uproot_loader():
             with uproot.open(path, **root_dir_options)[treepath] as root_tree:
@@ -215,7 +317,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
                 else:
                     branches_with_weights = branches
                 branches_with_weights = tuple(branches_with_weights)
-                data = root_tree.arrays(expressions=branches_with_weights, library='pd')
+                data = root_tree.arrays(expressions=branches_with_weights, library="pd")
             data_np = data[branches].values
             if weights_are_branch:
                 weights_np = data[weights]
@@ -228,58 +330,77 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
             weights_np = weights
         dataset = LightDataset.from_tensor(data)
 
-        # dataset = dataset.repeat()
-        obs = [branches_alias.get(branch, branch) for branch in branches]
-        return Data(dataset=dataset, obs=obs, weights=weights_np, name=name, dtype=dtype)
+        return Data(  # *not* class, if subclass, keep constructor
+            dataset=dataset,
+            obs=obs,
+            weights=weights_np,
+            name=name,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
 
     @classmethod
-    def from_pandas(cls, df: pd.DataFrame, obs: ztyping.ObsTypeInput = None, weights: ztyping.WeightsInputType = None,
-                    name: str = None, dtype: tf.DType = None):
-        """Create a `Data` from a pandas DataFrame. If `obs` is `None`, columns are used as obs.
+    def from_numpy(
+        cls,
+        obs: ztyping.ObsTypeInput,
+        array: np.ndarray,
+        weights: ztyping.WeightsInputType = None,
+        name: str = None,
+        dtype: tf.DType = None,
+        use_hash=None,
+    ):
+        """Create ``Data`` from a ``np.array``.
 
         Args:
-            df:
-            weights: Weights of the data. Has to be 1-D and match the shape
-                of the data (nevents).
-            obs:
-            name:
-        """
-        if obs is None:
-            obs = list(df.columns)
-        array = df.values
-        return cls.from_numpy(obs=obs, array=array, weights=weights, name=name, dtype=dtype)
-
-    @classmethod
-    def from_numpy(cls, obs: ztyping.ObsTypeInput, array: np.ndarray, weights: ztyping.WeightsInputType = None,
-                   name: str = None, dtype: tf.DType = None):
-        """Create `Data` from a `np.array`.
-
-        Args:
-            obs:
-            array:
-            name:
+            obs: Observables of the data. They will be matched to the data in the same order.
+            array: Numpy array containing the data.
+            weights: Weights of the data. Has to be 1-D and match the shape of the data (nevents).
+            name: Name of the data.
+            dtype: dtype of the data.
+            use_hash: If ``True``, a hash of the data is created and is used to identify it in caching.
 
         Returns:
+            ``zfit.Data``: A ``Data`` object containing the unbinned data.
         """
 
-        if not isinstance(array, (np.ndarray)) and not (tf.is_tensor(array) and hasattr(array, 'numpy')):
-            raise TypeError(f"`array` has to be a `np.ndarray`. Is currently {type(array)}")
+        if not isinstance(array, (np.ndarray)) and not (
+            tf.is_tensor(array) and hasattr(array, "numpy")
+        ):
+            raise TypeError(
+                f"`array` has to be a `np.ndarray`. Is currently {type(array)}"
+            )
         if dtype is None:
             dtype = ztypes.float
         tensor = tf.cast(array, dtype=dtype)
-        return cls.from_tensor(obs=obs, tensor=tensor, weights=weights, name=name, dtype=dtype)
+        return Data.from_tensor(  # *not* class, if subclass, keep constructor
+            obs=obs,
+            tensor=tensor,
+            weights=weights,
+            name=name,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
 
     @classmethod
-    def from_tensor(cls, obs: ztyping.ObsTypeInput, tensor: tf.Tensor, weights: ztyping.WeightsInputType = None,
-                    name: str = None, dtype: tf.DType = None) -> "Data":
-        """Create a `Data` from a `tf.Tensor`. `Value` simply returns the tensor (in the right order).
+    def from_tensor(
+        cls,
+        obs: ztyping.ObsTypeInput,
+        tensor: tf.Tensor,
+        weights: ztyping.WeightsInputType = None,
+        name: str = None,
+        dtype: tf.DType = None,
+        use_hash=None,
+    ) -> Data:
+        """Create a ``Data`` from a ``tf.Tensor``. ``Value`` simply returns the tensor (in the right order).
 
         Args:
-            obs:
-            tensor:
-            name:
+            obs: Observables of the data. They will be matched to the data in the same order.
+            tensor: Tensor containing the data.
+            weights: Weights of the data. Has to be 1-D and match the shape of the data (nevents).
+            name: Name of the data.
 
         Returns:
+            ``zfit.Data``: A ``Data`` object containing the unbinned data.
         """
         # dataset = LightDataset.from_tensor(tensor=tensor)
         if dtype is None:
@@ -292,13 +413,38 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         # dataset = tf.data.Dataset.from_tensor_slices(tensor)
         dataset = LightDataset.from_tensor(tensor)
 
-        return Data(dataset=dataset, obs=obs, name=name, weights=weights, dtype=dtype)
+        return Data(  # *not* class, if subclass, keep constructor
+            dataset=dataset,
+            obs=obs,
+            name=name,
+            weights=weights,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
+
+    def _update_hash(self):
+        if not run.executing_eagerly() or not self._use_hash:
+            self._hashint = None
+        else:
+            try:
+                hashval = xxhash.xxh128(np.asarray(self.value()))
+                if self.has_weights:
+                    hashval.update(np.asarray(self.weights))
+                self._hashint = hashval.intdigest()
+            except AttributeError:  # if the dataset is not yet initialized; this is allowed
+                self._hashint = None
+
+    def with_obs(self, obs):
+        values = self.value(obs)
+        return type(self).from_tensor(
+            obs=self.space, tensor=values, weights=self.weights, name=self.name
+        )
 
     def to_pandas(self, obs: ztyping.ObsTypeInput = None):
-        """Create a `pd.DataFrame` from `obs` as columns and return it.
+        """Create a ``pd.DataFrame`` from ``obs`` as columns and return it.
 
         Args:
-            obs: The observables to use as columns. If `None`, all observables are used.
+            obs: The observables to use as columns. If ``None``, all observables are used.
 
         Returns:
         """
@@ -323,18 +469,7 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         return z.unstack_x(self.value(obs=obs))
 
     def value(self, obs: ztyping.ObsTypeInput = None):
-        return self._value_internal(obs=obs)
-        # TODO: proper iterations
-        # value_iter = self._value_internal(obs=obs)
-        # value = next(value_iter)
-        # try:
-        #     next(value_iter)
-        # except StopIteration:  # it's ok, we're not batched
-        #     return value
-        # else:
-        #     raise DataIsBatchedError(
-        #         f"Data {self} is batched, cannot return only the value. Iterate through it (WIP, make"
-        #         f"an issue on Github if this feature is needed now)")
+        return znp.asarray(self._value_internal(obs=obs))
 
     def numpy(self):
         return self.value().numpy()
@@ -369,16 +504,24 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
             value = tf.cast(value, dtype=self.dtype)
         return value
 
-    def _sort_value(self, value, obs: Tuple[str]):
+    def _sort_value(self, value, obs: tuple[str]):
         obs = convert_to_container(value=obs, container=tuple)
-        perm_indices = self.space.axes if self.space.axes != tuple(range(value.shape[-1])) else False
+        # TODO CURRENT: deactivated below!
+        perm_indices = (
+            self.space.axes
+            if self.space.axes != tuple(range(value.shape[-1]))
+            else False
+        )
 
         # permutate = perm_indices is not None
         if obs:
             if not frozenset(obs) <= frozenset(self.obs):
-                raise ValueError("The observable(s) {} are not contained in the dataset. "
-                                 "Only the following are: {}".format(frozenset(obs) - frozenset(self.obs),
-                                                                     self.obs))
+                raise ValueError(
+                    "The observable(s) {} are not contained in the dataset. "
+                    "Only the following are: {}".format(
+                        frozenset(obs) - frozenset(self.obs), self.obs
+                    )
+                )
             perm_indices = self.space.get_reorder_indices(obs=obs)
             # values = list(values[self.obs.index(o)] for o in obs if o in self.obs)
         if perm_indices:
@@ -394,9 +537,12 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
     def sort_by_axes(self, axes: ztyping.AxesTypeInput, allow_superset: bool = True):
         if not allow_superset:
             if not frozenset(axes) <= frozenset(self.axes):
-                raise ValueError("The observable(s) {} are not contained in the dataset. "
-                                 "Only the following are: {}".format(frozenset(axes) - frozenset(self.axes),
-                                                                     self.axes))
+                raise ValueError(
+                    "The observable(s) {} are not contained in the dataset. "
+                    "Only the following are: {}".format(
+                        frozenset(axes) - frozenset(self.axes), self.axes
+                    )
+                )
         space = self.space.with_axes(axes=axes, allow_subset=True)
 
         def setter(value):
@@ -411,11 +557,16 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
     def sort_by_obs(self, obs: ztyping.ObsTypeInput, allow_superset: bool = False):
         if not allow_superset:
             if not frozenset(obs) <= frozenset(self.obs):
-                raise ValueError("The observable(s) {} are not contained in the dataset. "
-                                 "Only the following are: {}".format(frozenset(obs) - frozenset(self.obs),
-                                                                     self.obs))
+                raise ValueError(
+                    "The observable(s) {} are not contained in the dataset. "
+                    "Only the following are: {}".format(
+                        frozenset(obs) - frozenset(self.obs), self.obs
+                    )
+                )
 
-        space = self.space.with_obs(obs=obs, allow_subset=True, allow_superset=allow_superset)
+        space = self.space.with_obs(
+            obs=obs, allow_subset=True, allow_superset=allow_superset
+        )
 
         def setter(value):
             self._space = value
@@ -425,65 +576,23 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
 
         return TemporarilySet(value=space, setter=setter, getter=getter)
 
-    def _dense_var_to_tensor(self, dtype=None, name=None, as_ref=False):
-        del name
-        if dtype is not None:
-            if dtype != self.dtype:
-                return NotImplemented
-        if as_ref:
-            # return "NEVER READ THIS"
-            raise LogicalUndefinedOperationError("There is no ref for the `Data`")
-        else:
-            return self.value()
-
-    def _AsTensor(self):
-        return self.value()
-
-    @staticmethod
-    def _OverloadAllOperators():  # pylint: disable=invalid-name
-        """Register overloads for all operators."""
-        for operator in tf.Tensor.OVERLOADABLE_OPERATORS:
-            Data._OverloadOperator(operator)
-        # For slicing, bind getitem differently than a tensor (use SliceHelperVar
-        # instead)
-        # pylint: disable=protected-access
-        setattr(Data, "__getitem__", array_ops._SliceHelperVar)
-
-    @staticmethod
-    def _OverloadOperator(operator):  # pylint: disable=invalid-name
-        """Defer an operator overload to `ops.Tensor`.
-
-        We pull the operator out of ops.Tensor dynamically to avoid ordering issues.
-        Args:
-          operator: string. The operator name.
-        """
-
-        tensor_oper = getattr(tf.Tensor, operator)
-
-        def _run_op(a, *args):
-            # pylint: disable=protected-access
-            value = a._AsTensor()
-            return tensor_oper(value, *args)
-
-        # Propagate __doc__ to wrapper
-        try:
-            _run_op.__doc__ = tensor_oper.__doc__
-        except AttributeError:
-            pass
-
-        setattr(Data, operator, _run_op)
-
     def _check_input_data_range(self, data_range):
-        data_range = self.convert_sort_space(limits=data_range)
+        data_range = self._convert_sort_space(limits=data_range)
         if not frozenset(self.data_range.obs) == frozenset(data_range.obs):
-            raise ObsIncompatibleError(f"Data range has to cover the full observable space {self.data_range.obs}, not "
-                                       f"only {data_range.obs}")
+            raise ObsIncompatibleError(
+                f"Data range has to cover the full observable space {self.data_range.obs}, not "
+                f"only {data_range.obs}"
+            )
         return data_range
 
     # TODO(Mayou36): refactor with pdf or other range things?
-    def convert_sort_space(self, obs: ztyping.ObsTypeInput = None, axes: ztyping.AxesTypeInput = None,
-                           limits: ztyping.LimitsTypeInput = None) -> Union[Space, None]:
-        """Convert the inputs (using eventually `obs`, `axes`) to
+    def _convert_sort_space(
+        self,
+        obs: ztyping.ObsTypeInput = None,
+        axes: ztyping.AxesTypeInput = None,
+        limits: ztyping.LimitsTypeInput = None,
+    ) -> Space | None:
+        """Convert the inputs (using eventually ``obs``, ``axes``) to
         :py:class:`~zfit.Space` and sort them according to own `obs`.
 
         Args:
@@ -505,16 +614,34 @@ class Data(GraphCachable, ZfitData, BaseDimensional, BaseObject):
         return tf.shape(input=self.value())[0]
 
     def __str__(self) -> str:
-        return f'<zfit.Data: {self.name} obs={self.obs}>'
+        return f"<zfit.Data: {self.name} obs={self.obs}>"
+
+    def to_binned(self, space):
+        from zfit._data.binneddatav1 import BinnedData
+
+        return BinnedData.from_unbinned(space=space, data=self)
 
 
 class SampleData(Data):
     _cache_counting = 0
 
-    def __init__(self, dataset: Union[tf.data.Dataset, "LightDataset"],
-                 obs: ztyping.ObsTypeInput = None, weights=None, name: str = None,
-                 dtype: tf.DType = ztypes.float):
-        super().__init__(dataset, obs, name=name, weights=weights, iterator_feed_dict=None, dtype=dtype)
+    def __init__(
+        self,
+        dataset: tf.data.Dataset | LightDataset,
+        obs: ztyping.ObsTypeInput = None,
+        weights=None,
+        name: str = None,
+        dtype: tf.DType = ztypes.float,
+        use_hash: bool = None,
+    ):
+        super().__init__(
+            dataset,
+            obs,
+            name=name,
+            weights=weights,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
 
     @classmethod
     def get_cache_counting(cls):
@@ -523,26 +650,51 @@ class SampleData(Data):
         return counting
 
     @classmethod
-    def from_sample(cls, sample: tf.Tensor, obs: ztyping.ObsTypeInput, name: str = None,
-                    weights=None):
+    def from_sample(
+        cls,
+        sample: tf.Tensor,
+        obs: ztyping.ObsTypeInput,
+        name: str = None,
+        weights=None,
+        use_hash: bool = None,
+    ):
         dataset = LightDataset.from_tensor(sample)
-        return SampleData(dataset=dataset, obs=obs, name=name, weights=weights)
+        return SampleData(
+            dataset=dataset, obs=obs, name=name, weights=weights, use_hash=use_hash
+        )
 
 
 class Sampler(Data):
     _cache_counting = 0
 
-    def __init__(self, dataset: "LightDataset", sample_func: Callable, sample_holder: tf.Variable,
-                 n: Union[ztyping.NumericalScalarType, Callable], weights=None,
-                 fixed_params: Dict["zfit.Parameter", ztyping.NumericalScalarType] = None,
-                 obs: ztyping.ObsTypeInput = None, name: str = None,
-                 dtype: tf.DType = ztypes.float):
+    def __init__(
+        self,
+        dataset: LightDataset,
+        sample_func: Callable,
+        sample_holder: tf.Variable,
+        n: ztyping.NumericalScalarType | Callable,
+        weights=None,
+        fixed_params: dict[zfit.Parameter, ztyping.NumericalScalarType] = None,
+        obs: ztyping.ObsTypeInput = None,
+        name: str = None,
+        dtype: tf.DType = ztypes.float,
+        use_hash: bool = None,
+    ):
 
-        super().__init__(dataset=dataset, obs=obs, name=name, weights=weights, iterator_feed_dict=None, dtype=dtype)
+        super().__init__(
+            dataset=dataset,
+            obs=obs,
+            name=name,
+            weights=weights,
+            dtype=dtype,
+            use_hash=use_hash,
+        )
         if fixed_params is None:
             fixed_params = OrderedDict()
         if isinstance(fixed_params, (list, tuple)):
-            fixed_params = OrderedDict((param, param.numpy()) for param in fixed_params)  # TODO: numpy -> read_value?
+            fixed_params = OrderedDict(
+                (param, param.numpy()) for param in fixed_params
+            )  # TODO: numpy -> read_value?
 
         self._initial_resampled = False
 
@@ -568,9 +720,15 @@ class Sampler(Data):
         if not self._initial_resampled:
             raise RuntimeError(
                 "No data generated yet. Use `resample()` to generate samples or directly use `model.sample()`"
-                "for single-time sampling.")
+                "for single-time sampling."
+            )
         return super()._value_internal(obs=obs, filter=filter)
 
+    @property
+    def hashint(self) -> int | None:
+        return None  # since the variable can be changed but this may stays static... and using 128 bits we can't have
+        # a tf.Variable that keeps the int
+
     @classmethod
     def get_cache_counting(cls):
         counting = cls._cache_counting
@@ -578,8 +736,17 @@ class Sampler(Data):
         return counting
 
     @classmethod
-    def from_sample(cls, sample_func: Callable, n: ztyping.NumericalScalarType, obs: ztyping.ObsTypeInput,
-                    fixed_params=None, name: str = None, weights=None, dtype=None):
+    def from_sample(
+        cls,
+        sample_func: Callable,
+        n: ztyping.NumericalScalarType,
+        obs: ztyping.ObsTypeInput,
+        fixed_params=None,
+        name: str = None,
+        weights=None,
+        dtype=None,
+        use_hash: bool = None,
+    ):
         obs = convert_to_space(obs)
 
         if fixed_params is None:
@@ -587,21 +754,34 @@ class Sampler(Data):
         if dtype is None:
             dtype = ztypes.float
         # from tensorflow.python.ops.variables import VariableV1
-        sample_holder = tf.Variable(initial_value=sample_func(), dtype=dtype, trainable=False,  # HACK: sample_func
-                                    # validate_shape=False,
-                                    shape=(None, obs.n_obs),
-                                    name=f"sample_data_holder_{cls.get_cache_counting()}")
+        sample_holder = tf.Variable(
+            initial_value=sample_func(),
+            dtype=dtype,
+            trainable=False,
+            # validate_shape=False,
+            shape=(None, obs.n_obs),
+            name=f"sample_data_holder_{cls.get_cache_counting()}",
+        )
         dataset = LightDataset.from_tensor(sample_holder)
 
-        return Sampler(dataset=dataset, sample_holder=sample_holder, sample_func=sample_func, fixed_params=fixed_params,
-                       n=n, obs=obs, name=name, weights=weights)
+        return cls(
+            dataset=dataset,
+            sample_holder=sample_holder,
+            sample_func=sample_func,
+            fixed_params=fixed_params,
+            n=n,
+            obs=obs,
+            name=name,
+            weights=weights,
+            use_hash=use_hash,
+        )
 
-    def resample(self, param_values: Mapping = None, n: Union[int, tf.Tensor] = None):
+    def resample(self, param_values: Mapping = None, n: int | tf.Tensor = None):
         """Update the sample by newly sampling. This affects any object that used this data already.
 
-        All params that are not in the attribute `fixed_params` will use their current value for
+        All params that are not in the attribute ``fixed_params`` will use their current value for
         the creation of the new sample. The value can also be overwritten for one sampling by providing
-        a mapping with `param_values` from `Parameter` to the temporary `value`.
+        a mapping with ``param_values`` from ``Parameter`` to the temporary ``value``.
 
         Args:
             param_values: a mapping from :py:class:`~zfit.Parameter` to a `value`. For the current sampling,
@@ -618,7 +798,10 @@ class Sampler(Data):
 
         with ExitStack() as stack:
 
-            _ = [stack.enter_context(param.set_value(val)) for param, val in temp_param_values.items()]
+            _ = [
+                stack.enter_context(param.set_value(val))
+                for param, val in temp_param_values.items()
+            ]
 
             # if not (n and self._initial_resampled):  # we want to load and make sure that it's initialized
             #     # means it's handled inside the function
@@ -635,16 +818,16 @@ class Sampler(Data):
             # self.sample_holder.assign(new_sample)
             self.sample_holder.assign(new_sample, read_value=False)
             self._initial_resampled = True
+        self._update_hash()
 
     def __str__(self) -> str:
-        return f'<Sampler: {self.name} obs={self.obs}>'
+        return f"<Sampler: {self.name} obs={self.obs}>"
 
 
-register_tensor_conversion(Data, name='Data', overload_operators=True)
+register_tensor_conversion(Data, name="Data", overload_operators=True)
 
 
 class LightDataset:
-
     def __init__(self, tensor):
         if not isinstance(tensor, (tf.Tensor, tf.Variable)):
             tensor = z.convert_to_tensor(tensor)
@@ -664,7 +847,12 @@ class LightDataset:
         return self.tensor
 
 
-def sum_samples(sample1: ZfitData, sample2: ZfitData, obs: ZfitSpace, shuffle: bool = False):
+def sum_samples(
+    sample1: ZfitUnbinnedData,
+    sample2: ZfitUnbinnedData,
+    obs: ZfitSpace,
+    shuffle: bool = False,
+):
     samples = [sample1, sample2]
     if obs is None:
         raise WorkInProgressError
