@@ -1,23 +1,24 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
-from typing import Dict, Iterable, List, Set, Union
+from __future__ import annotations
+
+from collections.abc import Iterable
 
 import numpy as np
 
-import zfit
-from zfit.util.exception import SpaceIncompatibleError
-
+from .interfaces import ZfitDimensional
+from ..util.exception import SpaceIncompatibleError
 from ..util import ztyping
 from ..util.container import convert_to_container
-from .interfaces import ZfitDimensional
 
 
 class BaseDimensional(ZfitDimensional):
-
     def _check_n_obs(self, space):
         if self._N_OBS is not None:
             if len(space.obs) != self._N_OBS:
-                raise SpaceIncompatibleError(f"Exactly {self._N_OBS} obs are allowed, {space.obs} are given.")
+                raise SpaceIncompatibleError(
+                    f"Exactly {self._N_OBS} obs are allowed, {space.obs} are given."
+                )
 
     @classmethod
     def __init_subclass__(cls, **kwargs):
@@ -41,7 +42,7 @@ class BaseDimensional(ZfitDimensional):
 def get_same_obs(obs):
     deps = [set() for _ in range(len(obs))]
     for i, ob in enumerate(obs):
-        for j, other_ob in enumerate(obs[i + 1:]):
+        for j, other_ob in enumerate(obs[i + 1 :]):
             if not set(ob).isdisjoint(other_ob):
                 deps[i].add(i)
                 deps[i].add(j + i + 1)
@@ -52,10 +53,12 @@ def get_same_obs(obs):
     return deps
 
 
-def limits_overlap(spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bool = False) -> bool:
-    """Check if _any_ of the limits of `spaces` overlaps with _any_ other of `spaces`.
+def limits_overlap(
+    spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bool = False
+) -> bool:
+    """Check if _any_ of the limits of ``spaces`` overlaps with _any_ other of ``spaces``.
 
-    This also checks multiple limits within one space. If `allow_exact_match` is set to true, then
+    This also checks multiple limits within one space. If ``allow_exact_match`` is set to true, then
     an *exact* overlap of limits is allowed.
 
 
@@ -63,7 +66,7 @@ def limits_overlap(spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bo
         spaces:
         allow_exact_match: An exact overlap of two limits is counted as "not overlapping".
             Example: limits from -1 to 3 and 4 to 5 to *NOT* overlap with the limits 4 to 5 *iff*
-            `allow_exact_match` is True.
+            ``allow_exact_match`` is True.
 
     Returns:
         If there are overlapping limits.
@@ -87,11 +90,19 @@ def limits_overlap(spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bo
                 up = upper[:, index]
 
                 for other_lower, other_upper in zip(lowers, uppers):
-                    if allow_exact_match and np.allclose(other_lower, low) and np.allclose(other_upper, up):
+                    if (
+                        allow_exact_match
+                        and np.allclose(other_lower, low)
+                        and np.allclose(other_upper, up)
+                    ):
                         continue
                     # TODO(Mayou36): tol? add global flags?
-                    low_overlaps = np.all(other_lower - eps < low) and np.all(low < other_upper - eps)
-                    up_overlaps = np.all(other_lower + eps < up) and np.all(up < other_upper + eps)
+                    low_overlaps = np.all(other_lower - eps < low) and np.all(
+                        low < other_upper - eps
+                    )
+                    up_overlaps = np.all(other_lower + eps < up) and np.all(
+                        up < other_upper + eps
+                    )
                     overlap = low_overlaps or up_overlaps
                     if overlap:
                         return True
@@ -100,8 +111,8 @@ def limits_overlap(spaces: ztyping.SpaceOrSpacesTypeInput, allow_exact_match: bo
     return False
 
 
-def common_obs(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool]:
-    """Extract the union of `obs` from `spaces` in the order of `spaces`.
+def common_obs(spaces: ztyping.SpaceOrSpacesTypeInput) -> list[str] | bool:
+    """Extract the union of ``obs`` from ``spaces`` in the order of ``spaces``.
 
     For example:
         | space1.obs: ['obs1', 'obs3']
@@ -114,7 +125,7 @@ def common_obs(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool]
         spaces: :py:class:`~zfit.Space`s to extract the obs from
 
     Returns:
-        The observables as `str` or False if not every space has observables
+        The observables as ``str`` or False if not every space has observables
     """
     spaces = convert_to_container(spaces, container=tuple)
     all_obs = []
@@ -127,8 +138,8 @@ def common_obs(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool]
     return all_obs
 
 
-def common_axes(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool]:
-    """Extract the union of `axes` from `spaces` in the order of `spaces`.
+def common_axes(spaces: ztyping.SpaceOrSpacesTypeInput) -> list[str] | bool:
+    """Extract the union of ``axes`` from ``spaces`` in the order of ``spaces``.
 
     For example:
         | space1.axes: [1, 3]
@@ -154,8 +165,10 @@ def common_axes(spaces: ztyping.SpaceOrSpacesTypeInput) -> Union[List[str], bool
     return all_axes
 
 
-def obs_subsets(dimensionals: Iterable[ZfitDimensional]) -> Dict[Set[str], ZfitDimensional]:
-    """Split `dimensionals` into the smallest subgroup of obs and return a dict.
+def obs_subsets(
+    dimensionals: Iterable[ZfitDimensional],
+) -> dict[set[str], ZfitDimensional]:
+    """Split ``dimensionals`` into the smallest subgroup of obs and return a dict.
 
     Args:
         dimensionals: An Iterable containing two or more ZfitDimensional that should be split into the smallest subset.
