@@ -1,11 +1,19 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
+
+from collections.abc import Callable
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    import zfit
 
 import abc
 from abc import ABCMeta, abstractmethod
-from typing import Callable, Dict, List, Optional, Set, Tuple, Union
 
 import numpy as np
 import tensorflow as tf
+from uhi.typing.plottable import PlottableHistogram
 
 from ..util import ztyping
 from ..util.deprecation import deprecated
@@ -19,7 +27,6 @@ class ZfitObject(abc.ABC):
 
 
 class ZfitDimensional(ZfitObject):
-
     @property
     @abstractmethod
     def obs(self) -> ztyping.ObsTypeReturn:
@@ -41,15 +48,22 @@ class ZfitDimensional(ZfitObject):
         """
         raise NotImplementedError
 
+    # TODO: activate?
+    # @property
+    # @abstractmethod
+    # def space(self):
+    #     raise NotImplementedError
+
 
 class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
-
     @abstractmethod
-    def with_obs(self,
-                 obs: Optional[ztyping.ObsTypeInput],
-                 allow_superset: bool = True,
-                 allow_subset: bool = True) -> "ZfitOrderableDimensional":
-        """Create a new instance that has `obs`; sorted by or set or dropped.
+    def with_obs(
+        self,
+        obs: ztyping.ObsTypeInput | None,
+        allow_superset: bool = True,
+        allow_subset: bool = True,
+    ) -> ZfitOrderableDimensional:
+        """Create a new instance that has ``obs``; sorted by or set or dropped.
 
         The behavior is as follows:
 
@@ -79,17 +93,19 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
 
         Raises:
             CoordinatesUnderdefinedError: if obs is None and the instance does not have axes
-            ObsIncompatibleError: if `obs` is a superset and allow_superset is False or a subset and
+            ObsIncompatibleError: if ``obs`` is a superset and allow_superset is False or a subset and
                 allow_allow_subset is False
         """
         raise NotImplementedError
 
     @abstractmethod
-    def with_axes(self,
-                  axes: Optional[ztyping.AxesTypeInput],
-                  allow_superset: bool = True,
-                  allow_subset: bool = True) -> "ZfitOrderableDimensional":
-        """Create a new instance that has `axes`; sorted by or set or dropped.
+    def with_axes(
+        self,
+        axes: ztyping.AxesTypeInput | None,
+        allow_superset: bool = True,
+        allow_subset: bool = True,
+    ) -> ZfitOrderableDimensional:
+        """Create a new instance that has ``axes``; sorted by or set or dropped.
 
         The behavior is as follows:
 
@@ -119,18 +135,18 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
 
         Raises:
             CoordinatesUnderdefinedError: if obs is None and the instance does not have axes
-            AxesIncompatibleError: if `axes` is a superset and allow_superset is False or a subset and
+            AxesIncompatibleError: if ``axes`` is a superset and allow_superset is False or a subset and
                 allow_allow_subset is False
         """
 
         raise NotImplementedError
 
     @abstractmethod
-    def with_autofill_axes(self, overwrite: bool = False) -> "ZfitOrderableDimensional":
+    def with_autofill_axes(self, overwrite: bool = False) -> ZfitOrderableDimensional:
         """Overwrite the axes of the current object with axes corresponding to range(len(n_obs)).
 
         This effectively fills with (0, 1, 2,...) and can be used mostly when an object enters a PDF or
-        similar. `overwrite` allows to remove the axis first in case there are already some set.
+        similar. ``overwrite`` allows to remove the axis first in case there are already some set.
 
         .. code-block::
 
@@ -145,34 +161,36 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
 
         Args:
             overwrite: If axes are already set, replace the axes with the autofilled ones.
-                If axes is already set and `overwrite` is False, raise an error.
+                If axes is already set and ``overwrite`` is False, raise an error.
 
         Returns:
             The object with the new axes
 
         Raises:
-            AxesIncompatibleError: if the axes are already set and `overwrite` is False.
+            AxesIncompatibleError: if the axes are already set and ``overwrite`` is False.
         """
         raise NotImplementedError
 
     @abstractmethod
-    def reorder_x(self, x: Union[tf.Tensor, np.ndarray],
-                  *,
-                  x_obs: ztyping.ObsTypeInput = None,
-                  x_axes: ztyping.AxesTypeInput = None,
-                  func_obs: ztyping.ObsTypeInput = None,
-                  func_axes: ztyping.AxesTypeInput = None
-                  ) -> ztyping.XTypeReturnNoData:
+    def reorder_x(
+        self,
+        x: tf.Tensor | np.ndarray,
+        *,
+        x_obs: ztyping.ObsTypeInput = None,
+        x_axes: ztyping.AxesTypeInput = None,
+        func_obs: ztyping.ObsTypeInput = None,
+        func_axes: ztyping.AxesTypeInput = None,
+    ) -> ztyping.XTypeReturnNoData:
         """Reorder x in the last dimension either according to its own obs or assuming a function ordered with func_obs.
 
         There are two obs or axes around: the one associated with this Coordinate object and the one associated with x.
         If x_obs or x_axes is given, then this is assumed to be the obs resp. the axes of x and x will be reordered
-        according to `self.obs` resp. `self.axes`.
+        according to ``self.obs`` resp. ``self.axes``.
 
-        If func_obs resp. func_axes is given, then x is assumed to have `self.obs` resp. `self.axes` and will be
-        reordered to align with a function ordered with `func_obs` resp. `func_axes`.
+        If func_obs resp. func_axes is given, then x is assumed to have ``self.obs`` resp. ``self.axes`` and will be
+        reordered to align with a function ordered with ``func_obs`` resp. ``func_axes``.
 
-        Switching `func_obs` for `x_obs` resp. `func_axes` for `x_axes` inverts the reordering of x.
+        Switching ``func_obs`` for ``x_obs`` resp. ``func_axes`` for ``x_axes`` inverts the reordering of x.
 
         Args:
             x: Tensor to be reordered, last dimension should be n_obs resp. n_axes
@@ -180,7 +198,7 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
                 latter.
             x_axes: Axes associated with x.
             func_obs: Observables associated with a function that x will be given to. Reorders x accordingly and assumes
-                self.obs to be the obs of x. If both, `func_obs` and `func_axes` are given, this has precedency over the
+                self.obs to be the obs of x. If both, ``func_obs`` and ``func_axes`` are given, this has precedency over the
                 latter.
             func_axes: Axe associated with a function that x will be given to. Reorders x accordingly and assumes
                 self.axes to be the axes of x.
@@ -191,11 +209,10 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
         raise NotImplementedError
 
     @abstractmethod
-    def get_reorder_indices(self,
-                            obs: ztyping.ObsTypeInput = None,
-                            axes: ztyping.AxesTypeInput = None
-                            ) -> Tuple[int]:
-        """Indices that would order the instances obs as `obs` respectively the instances axes as `axes`.
+    def get_reorder_indices(
+        self, obs: ztyping.ObsTypeInput = None, axes: ztyping.AxesTypeInput = None
+    ) -> tuple[int]:
+        """Indices that would order the instances obs as ``obs`` respectively the instances axes as ``axes``.
 
         Args:
             obs: Observables that the instances obs should be ordered to. Does not reorder, but just
@@ -207,15 +224,14 @@ class ZfitOrderableDimensional(ZfitDimensional, metaclass=ABCMeta):
             New indices that would reorder the instances obs to be obs respectively axes.
 
         Raises:
-            CoordinatesUnderdefinedError: If neither `obs` nor `axes` is given
+            CoordinatesUnderdefinedError: If neither ``obs`` nor ``axes`` is given
         """
         raise NotImplementedError
 
 
 class ZfitData(ZfitDimensional):
-
     @abstractmethod
-    def value(self, obs: List[str] = None) -> ztyping.XType:
+    def value(self, obs: list[str] = None) -> ztyping.XType:
         raise NotImplementedError
 
     @abstractmethod
@@ -232,19 +248,22 @@ class ZfitData(ZfitDimensional):
         raise NotImplementedError
 
 
-class ZfitLimit(abc.ABC, metaclass=ABCMeta):
+class ZfitUnbinnedData(ZfitData):
+    pass
 
+
+class ZfitLimit(abc.ABC, metaclass=ABCMeta):
     @property
     @abstractmethod
     def rect_limits(self) -> ztyping.RectLimitsReturnType:
-        """Return the rectangular limits as `np.ndarray``tf.Tensor` if they are set and not false.
+        """Return the rectangular limits as ``np.ndarray``tf.Tensor`` if they are set and not false.
 
             The rectangular limits can be used for sampling. They do not in general represent the limits
             of the object as a functional limit can be set and to check if something is inside the limits,
             the method :py:meth:`~Limit.inside` should be used.
 
             In order to test if the limits are False or None, it is recommended to use the appropriate methods
-            `limits_are_false` and `limits_are_set`.
+            ``limits_are_false`` and ``limits_are_set``.
 
         Returns:
             The lower and upper limits.
@@ -256,19 +275,19 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
     @property
     @abstractmethod
     def rect_limits_np(self) -> ztyping.RectLimitsNPReturnType:
-        """Return the rectangular limits as `np.ndarray`. Raise error if not possible.
+        """Return the rectangular limits as ``np.ndarray``. Raise error if not possible.
 
         Rectangular limits are returned as numpy arrays which can be useful when doing checks that do not
         need to be involved in the computation later on as they allow direct interaction with Python as
-        compared to `tf.Tensor` inside a graph function.
+        compared to ``tf.Tensor`` inside a graph function.
 
         In order to test if the limits are False or None, it is recommended to use the appropriate methods
-        `limits_are_false` and `limits_are_set`.
+        ``limits_are_false`` and ``limits_are_set``.
 
         Returns:
-            A tuple of two `np.ndarray` with shape (1, n_obs) typically. The last
-                dimension is always `n_obs`, the first can be vectorized. This allows unstacking
-                with `z.unstack_x()` as can be done with data.
+            A tuple of two ``np.ndarray`` with shape (1, n_obs) typically. The last
+                dimension is always ``n_obs``, the first can be vectorized. This allows unstacking
+                with ``z.unstack_x()`` as can be done with data.
 
         Raises:
             CannotConvertToNumpyError: In case the conversion fails.
@@ -279,10 +298,10 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
     @property
     @abstractmethod
     def rect_lower(self) -> ztyping.RectLowerReturnType:
-        """The lower, rectangular limits, equivalent to `rect_limits[0]` with shape (..., n_obs)
+        """The lower, rectangular limits, equivalent to ``rect_limits[0]`` with shape (..., n_obs)
 
         Returns:
-            The lower, rectangular limits as `np.ndarray` or `tf.Tensor`
+            The lower, rectangular limits as ``np.ndarray`` or ``tf.Tensor``
         Raises:
             LimitsNotSpecifiedError: If the limits are not set or are false
         """
@@ -291,17 +310,17 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
     @property
     @abstractmethod
     def rect_upper(self) -> ztyping.RectUpperReturnType:
-        """The upper, rectangular limits, equivalent to `rect_limits[1]` with shape (..., n_obs)
+        """The upper, rectangular limits, equivalent to ``rect_limits[1]`` with shape (..., n_obs)
 
         Returns:
-            The upper, rectangular limits as `np.ndarray` or `tf.Tensor`
+            The upper, rectangular limits as ``np.ndarray`` or ``tf.Tensor``
         Raises:
             LimitsNotSpecifiedError: If the limits are not set or are false
         """
         raise NotImplementedError
 
     @abstractmethod
-    def rect_area(self) -> Union[float, np.ndarray, tf.Tensor]:
+    def rect_area(self) -> float | np.ndarray | tf.Tensor:
         """Calculate the total rectangular area of all the limits and axes.
 
         Useful, for example, for MC integration.
@@ -309,8 +328,10 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
         raise NotImplementedError
 
     @abstractmethod
-    def inside(self, x: ztyping.XTypeInput, guarantee_limits: bool = False) -> ztyping.XTypeReturn:
-        """Test if `x` is inside the limits.
+    def inside(
+        self, x: ztyping.XTypeInput, guarantee_limits: bool = False
+    ) -> ztyping.XTypeReturn:
+        """Test if ``x`` is inside the limits.
 
         This function should be used to test if values are inside the limits. If the given x is already inside
         the rectangular limits, e.g. because it was sampled from within them
@@ -321,28 +342,30 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
             guarantee_limits: Guarantee that the values are already inside the rectangular limits.
 
         Returns:
-            Return a boolean tensor-like object with the same shape as the input `x` except of the
+            Return a boolean tensor-like object with the same shape as the input ``x`` except of the
                 last dimension removed.
         """
         raise NotImplementedError
 
     @abstractmethod
-    def filter(self, x: ztyping.XTypeInput,
-               guarantee_limits: bool = False,
-               axis: Optional[int] = None
-               ) -> ztyping.XTypeReturnNoData:
-        """Filter `x` by removing the elements along `axis` that are not inside the limits.
+    def filter(
+        self,
+        x: ztyping.XTypeInput,
+        guarantee_limits: bool = False,
+        axis: int | None = None,
+    ) -> ztyping.XTypeReturnNoData:
+        """Filter ``x`` by removing the elements along ``axis`` that are not inside the limits.
 
-        This is similar to `tf.boolean_mask`.
+        This is similar to ``tf.boolean_mask``.
 
         Args:
             x: Values to be checked whether they are inside of the limits. If not, the corresonding element (in the
-                specified `axis`) is removed. The shape is expected to have the last dimension equal to n_obs.
+                specified ``axis``) is removed. The shape is expected to have the last dimension equal to n_obs.
             guarantee_limits: Guarantee that the values are already inside the rectangular limits.
             axis: The axis to remove the elements from. Defaults to 0.
 
         Returns:
-            Return an object with the same shape as `x` except that along `axis` elements have been
+            Return an object with the same shape as ``x`` except that along ``axis`` elements have been
                 removed.
         """
 
@@ -388,6 +411,7 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
     # TODO: remove from API?
     def get_subspace(self, *_, **__):
         from zfit.util.exception import InvalidLimitSubspaceError
+
         raise InvalidLimitSubspaceError("ZfitLimits does not suppoert subspaces")
 
     @property
@@ -401,7 +425,7 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
         raise NotImplementedError
 
     @property
-    def n_events(self) -> Union[int, None]:
+    def n_events(self) -> int | None:
         """Shape of the first dimension, usually reflects the number of events.
 
         Returns:
@@ -411,10 +435,10 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
         raise NotImplementedError
 
     @abstractmethod
-    def equal(self, other: object, allow_graph: bool) -> Union[bool, tf.Tensor]:
+    def equal(self, other: object, allow_graph: bool) -> bool | tf.Tensor:
         """Compare the limits on equality. For ANY objects, this also returns true.
 
-        If called inside a graph context *and* the limits are tensors, this will return a symbolic `tf.Tensor`.
+        If called inside a graph context *and* the limits are tensors, this will return a symbolic ``tf.Tensor``.
 
         Returns:
             Result of the comparison
@@ -433,15 +457,12 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
         raise NotImplementedError
 
     @abstractmethod
-    def less_equal(self,
-                   other: object,
-                   allow_graph: bool = True
-                   ) -> Union[bool, tf.Tensor]:
+    def less_equal(self, other: object, allow_graph: bool = True) -> bool | tf.Tensor:
         """Set-like comparison for compatibility. If an object is less_equal to another, the limits are combatible.
 
         This can be used to determine whether a fitting range specification can handle another limit.
 
-        If called inside a graph context *and* the limits are tensors, this will return a symbolic `tf.Tensor`.
+        If called inside a graph context *and* the limits are tensors, this will return a symbolic ``tf.Tensor``.
 
 
         Args:
@@ -481,6 +502,13 @@ class ZfitLimit(abc.ABC, metaclass=ABCMeta):
 
 
 class ZfitSpace(ZfitLimit, ZfitOrderableDimensional, ZfitObject, metaclass=ABCMeta):
+    @property
+    def is_binned(self):
+        raise NotImplementedError
+
+    @property
+    def binning(self):
+        raise NotImplementedError
 
     @property
     @abstractmethod
@@ -491,7 +519,7 @@ class ZfitSpace(ZfitLimit, ZfitOrderableDimensional, ZfitObject, metaclass=ABCMe
     # TODO: legacy?
     @property
     @abstractmethod
-    def limits(self) -> Tuple[ztyping.LowerTypeReturn, ztyping.UpperTypeReturn]:
+    def limits(self) -> tuple[ztyping.LowerTypeReturn, ztyping.UpperTypeReturn]:
         """Return the tuple(lower, upper)."""
         raise NotImplementedError
 
@@ -519,15 +547,16 @@ class ZfitSpace(ZfitLimit, ZfitOrderableDimensional, ZfitObject, metaclass=ABCMe
         raise NotImplementedError
 
     @abstractmethod
-    def with_limits(self,
-                    limits: ztyping.LimitsTypeInput = None,
-                    rect_limits: Optional[ztyping.RectLimitsInputType] = None,
-                    name: Optional[str] = None
-                    ) -> "ZfitSpace":
-        """Return a copy of the space with the new `limits` (and the new `name`).
+    def with_limits(
+        self,
+        limits: ztyping.LimitsTypeInput = None,
+        rect_limits: ztyping.RectLimitsInputType | None = None,
+        name: str | None = None,
+    ) -> ZfitSpace:
+        """Return a copy of the space with the new ``limits`` (and the new ``name``).
 
         Args:
-            limits: Limits to use. Can be rectangular, a function (requires to also specify `rect_limits`
+            limits: Limits to use. Can be rectangular, a function (requires to also specify ``rect_limits``
                 or an instance of ZfitLimit.
             rect_limits: Rectangular limits that will be assigned with the instance
             name: Human readable name
@@ -551,10 +580,12 @@ class ZfitSpace(ZfitLimit, ZfitOrderableDimensional, ZfitObject, metaclass=ABCMe
         raise NotImplementedError
 
     @abstractmethod
-    def with_coords(self,
-                    coords: ZfitOrderableDimensional,
-                    allow_superset: bool = True,
-                    allow_subset: bool = True) -> object:
+    def with_coords(
+        self,
+        coords: ZfitOrderableDimensional,
+        allow_superset: bool = True,
+        allow_subset: bool = True,
+    ) -> object:
         """Create a new :py:class:`~zfit.Space` with reordered observables and/or axes.
 
         The behavior is that _at least one coordinate (obs or axes) has to be set in both instances
@@ -593,9 +624,12 @@ class ZfitDependenciesMixin:
     def get_cache_deps(self, only_floating: bool = True) -> ztyping.DependentsType:
         raise NotImplementedError
 
-    @deprecated(date=None, instructions="Use `get_params` instead if you want to retrieve the "
-                                        "independent parameters or `get_cache_deps` in case you need "
-                                        "the numerical cache dependents (advanced).")
+    @deprecated(
+        date=None,
+        instructions="Use `get_params` instead if you want to retrieve the "
+        "independent parameters or `get_cache_deps` in case you need "
+        "the numerical cache dependents (advanced).",
+    )
     def get_dependencies(self, only_floating: bool = True) -> ztyping.DependentsType:
         # raise BreakingAPIChangeError
         return self.get_cache_deps(only_floating=only_floating)
@@ -603,11 +637,12 @@ class ZfitDependenciesMixin:
 
 class ZfitParametrized(ZfitDependenciesMixin, ZfitObject):
     @abstractmethod
-    def get_params(self,
-                   floating: Optional[bool] = True,
-                   is_yield: Optional[bool] = None,
-                   extract_independent: Optional[bool] = True
-                   ) -> Set["ZfitParameter"]:
+    def get_params(
+        self,
+        floating: bool | None = True,
+        is_yield: bool | None = None,
+        extract_independent: bool | None = True,
+    ) -> set[ZfitParameter]:
         """Recursively collect parameters that this object depends on according to the filter criteria.
 
         Which parameters should be included can be steered using the arguments as a filter.
@@ -635,7 +670,6 @@ class ZfitParametrized(ZfitDependenciesMixin, ZfitObject):
 
 
 class ZfitNumericParametrized(ZfitParametrized):
-
     @property
     @abstractmethod
     def dtype(self) -> tf.DType:
@@ -644,7 +678,6 @@ class ZfitNumericParametrized(ZfitParametrized):
 
 
 class ZfitParameter(ZfitNumericParametrized):
-
     @property
     @abstractmethod
     def name(self) -> str:
@@ -739,9 +772,8 @@ class ZfitIndependentParameter(ZfitParameter, metaclass=ABCMeta):
 
 
 class ZfitLoss(ZfitObject, metaclass=ABCMeta):
-
     @abstractmethod
-    def gradient(self, params: ztyping.ParamTypeInput = None) -> List[tf.Tensor]:
+    def gradient(self, params: ztyping.ParamTypeInput = None) -> list[tf.Tensor]:
         raise NotImplementedError
 
     @abstractmethod
@@ -750,21 +782,21 @@ class ZfitLoss(ZfitObject, metaclass=ABCMeta):
 
     @property
     @abstractmethod
-    def model(self) -> List["ZfitModel"]:
+    def model(self) -> list[ZfitModel]:
         raise NotImplementedError
 
     @property
     @abstractmethod
-    def data(self) -> List["ZfitData"]:
+    def data(self) -> list[ZfitData]:
         raise NotImplementedError
 
     @property
     @abstractmethod
-    def fit_range(self) -> List["ZfitSpace"]:
+    def fit_range(self) -> list[ZfitSpace]:
         raise NotImplementedError
 
     @abstractmethod
-    def add_constraints(self, constraints: List[tf.Tensor]):
+    def add_constraints(self, constraints: list[tf.Tensor]):
         raise NotImplementedError
 
     @property
@@ -789,19 +821,23 @@ class ZfitLoss(ZfitObject, metaclass=ABCMeta):
 
 
 class ZfitModel(ZfitNumericParametrized, ZfitDimensional):
-
     @abstractmethod
-    def update_integration_options(self, *args, **kwargs):  # TODO: handling integration properly
+    def update_integration_options(
+        self, *args, **kwargs
+    ):  # TODO: handling integration properly
         raise NotImplementedError
 
     @abstractmethod
-    def integrate(self, limits: ztyping.LimitsType, norm_range: ztyping.LimitsType = None,
-                  name: str = "integrate") -> ztyping.XType:
+    def integrate(
+        self, limits: ztyping.LimitsType, norm: ztyping.LimitsType = None, *, options
+    ) -> ztyping.XType:
         """Integrate the function over `limits` (normalized over `norm_range` if not False).
 
         Args:
+            * ():
+            options ():
             limits: the limits to integrate over
-            norm_range: the limits to normalize over or False to integrate the
+            norm: the limits to normalize over or False to integrate the
                 unnormalized probability
             name:
 
@@ -812,10 +848,15 @@ class ZfitModel(ZfitNumericParametrized, ZfitDimensional):
 
     @classmethod
     @abstractmethod
-    def register_analytic_integral(cls, func: Callable, limits: ztyping.LimitsType = None,
-                                   priority: int = 50, *,
-                                   supports_norm_range: bool = False,
-                                   supports_multiple_limits: bool = False):
+    def register_analytic_integral(
+        cls,
+        func: Callable,
+        limits: ztyping.LimitsType = None,
+        priority: int = 50,
+        *,
+        supports_norm: bool = False,
+        supports_multiple_limits: bool = False,
+    ):
         """Register an analytic integral with the class.
 
         Args:
@@ -823,21 +864,31 @@ class ZfitModel(ZfitNumericParametrized, ZfitDimensional):
             limits: |limits_arg_descr|
             priority:
             supports_multiple_limits:
-            supports_norm_range:
+            supports_norm:
 
         Returns:
         """
         raise NotImplementedError
 
     @abstractmethod
-    def partial_integrate(self, x: ztyping.XType, limits: ztyping.LimitsType,
-                          norm_range: ztyping.LimitsType = None) -> ztyping.XType:
+    def partial_integrate(
+        self,
+        x: ztyping.XType,
+        limits: ztyping.LimitsType,
+        *,
+        norm=None,
+        options=None,
+        norm_range: ztyping.LimitsType = None,
+    ) -> ztyping.XType:
         """Partially integrate the function over the `limits` and evaluate it at `x`.
 
         Dimension of `limits` and `x` have to add up to the full dimension and be therefore equal
         to the dimensions of `norm_range` (if not False)
 
         Args:
+            * ():
+            norm ():
+            options ():
             x: The value at which the partially integrated function will be evaluated
             limits: the limits to integrate over. Can contain only some axes
             norm_range: the limits to normalize over. Has to have all axes
@@ -883,9 +934,10 @@ class ZfitFunc(ZfitModel):
 
 
 class ZfitPDF(ZfitModel):
-
     @abstractmethod
-    def pdf(self, x: ztyping.XType, norm_range: ztyping.LimitsType = None) -> ztyping.XType:
+    def pdf(
+        self, x: ztyping.XType, norm: ztyping.LimitsType = None, norm_range=None
+    ) -> ztyping.XType:
         raise NotImplementedError
 
     @property
@@ -898,15 +950,17 @@ class ZfitPDF(ZfitModel):
         raise NotImplementedError
 
     @abstractmethod
-    def create_extended(self, yield_: ztyping.ParamTypeInput) -> "ZfitPDF":
+    def create_extended(self, yield_: ztyping.ParamTypeInput) -> ZfitPDF:
         raise NotImplementedError
 
     @abstractmethod
-    def get_yield(self) -> Union[ZfitParameter, None]:
+    def get_yield(self) -> ZfitParameter | None:
         raise NotImplementedError
 
     @abstractmethod
-    def normalization(self, limits: ztyping.LimitsType) -> ztyping.NumericalTypeReturn:
+    def normalization(
+        self, limits: ztyping.LimitsType, *, options
+    ) -> ztyping.NumericalTypeReturn:
         raise NotImplementedError
 
     @abstractmethod
@@ -915,14 +969,13 @@ class ZfitPDF(ZfitModel):
 
 
 class ZfitFunctorMixin:
-
     @property
     @abstractmethod
-    def models(self) -> Dict[Union[float, int, str], ZfitModel]:
+    def models(self) -> dict[float | int | str, ZfitModel]:
         raise NotImplementedError
 
     @abstractmethod
-    def get_models(self) -> List[ZfitModel]:
+    def get_models(self) -> list[ZfitModel]:
         raise NotImplementedError
 
 
@@ -930,3 +983,63 @@ class ZfitConstraint(abc.ABC):
     @abstractmethod
     def value(self):
         raise NotImplementedError
+
+
+class ZfitMinimalHist(PlottableHistogram):
+    @property
+    def kind(self):
+        raise NotImplementedError
+
+    def values(self):
+        raise NotImplementedError
+
+    def variances(self) -> np.ndarray | None:
+        raise NotImplementedError
+
+    def axes(self):
+        raise NotImplementedError
+
+
+class ZfitBinnedData(ZfitDimensional, ZfitMinimalHist, metaclass=ABCMeta):
+    @abstractmethod
+    def variances(self):
+        raise NotImplementedError
+
+    def with_obs(self, obs) -> ZfitBinnedData:
+        raise NotImplementedError
+
+    # @abstractmethod
+    # def counts(self):  # TODO: name?
+    #     raise NotImplementedError
+
+    # @abstractmethod
+    # def binning(self):
+    #     return self.space.binning
+    @abstractmethod
+    def to_hist(self):
+        """Convert the binned data to a :py:class:`~hist.NamedHist`.
+
+        While a binned data object can be used inside zfit (PDFs,...), it lacks many convenience features that the `hist
+        library <https://hist.readthedocs.io/>`_ offers, such as plots.
+        """
+        pass
+
+
+class ZfitBinnedPDF(ZfitPDF, metaclass=ABCMeta):
+    @abstractmethod
+    def counts(self, x, norm):
+        pass
+
+    @abstractmethod
+    def rel_counts(self, x, norm):
+        pass
+
+
+class ZfitBinning:
+    pass
+
+
+class ZfitRectBinning(ZfitBinning):
+    @abstractmethod
+    def get_edges(self):
+        raise NotImplementedError
