@@ -1,7 +1,7 @@
 """Module for caching.
 
 The basic concept of caching in Zfit builds on a "cacher", that caches a certain value and that
-is dependent of "cache_dependents". By implementing `ZfitCachable`, an object will be able to play both
+is dependent of "cache_dependents". By implementing `ZfitGraphCachable`, an object will be able to play both
 roles. And most importantly, it has a `_cache` dict, that contains all the cache.
 
 Basic principle
@@ -45,13 +45,17 @@ Example with a pdf that caches the normalization:
                 result = self._cache['my_name']
             return result
 """
+#  Copyright (c) 2022 zfit
 
-#  Copyright (c) 2021 zfit
+from __future__ import annotations
 
+import collections.abc
 import functools
 import weakref
 from abc import abstractmethod
-from typing import Iterable, Mapping, Union
+from collections.abc import Iterable
+from itertools import zip_longest
+from typing import Optional
 
 import numpy as np
 import tensorflow as tf
@@ -61,9 +65,8 @@ from .container import convert_to_container
 
 
 class ZfitGraphCachable:
-
     @abstractmethod
-    def register_cacher(self, cacher: "ZfitGraphCachable"):
+    def register_cacher(self, cacher: ZfitGraphCachable):
         raise NotImplementedError
 
     @abstractmethod
@@ -73,10 +76,10 @@ class ZfitGraphCachable:
         Args:
             cache_dependents:
             allow_non_cachable: If `True`, allow `cache_dependents` to be non-cachables.
-                If `False`, any `cache_dependents` that is not a `ZfitCachable` will raise an error.
+                If `False`, any `cache_dependents` that is not a `ZfitGraphCachable` will raise an error.
 
         Raises:
-            TypeError: if one of the `cache_dependents` is not a `ZfitCachable` _and_ `allow_non_cachable`
+            TypeError: if one of the `cache_dependents` is not a `ZfitGraphCachable` _and_ `allow_non_cachable`
                 if `False`.
         """
         pass
@@ -108,7 +111,7 @@ class GraphCachable(ZfitGraphCachable):
         for func_name in dir(cls):
             if not func_name.startswith("__"):
                 func = getattr(cls, func_name)
-                if callable(func) and hasattr(func, 'zfit_graph_cache_registered'):
+                if callable(func) and hasattr(func, "zfit_graph_cache_registered"):
                     # assert hasattr(func, "_descriptor_cache"), "TensorFlow internals have changed. Need to update cache"
                     func.zfit_graph_cache_registered = True
                     graph_caching_methods.append(func)
@@ -121,20 +124,22 @@ class GraphCachable(ZfitGraphCachable):
             cacher:
         """
         if not isinstance(cacher, ZfitGraphCachable):
-            raise TypeError(f"`cacher` is not a `ZfitCachable` but {type(cacher)}")
+            raise TypeError(f"`cacher` is not a `ZfitGraphCachable` but {type(cacher)}")
         if not cacher in self._cachers:
             self._cachers[cacher] = None  # could we have a more useful value?
 
-    def add_cache_deps(self, cache_deps: ztyping.CacherOrCachersType, allow_non_cachable: bool = True):
+    def add_cache_deps(
+        self, cache_deps: ztyping.CacherOrCachersType, allow_non_cachable: bool = True
+    ):
         """Add dependencies that render the cache invalid if they change.
 
         Args:
             cache_deps:
             allow_non_cachable: If `True`, allow `cache_dependents` to be non-cachables.
-                If `False`, any `cache_dependents` that is not a `ZfitCachable` will raise an error.
+                If `False`, any `cache_dependents` that is not a `ZfitGraphCachable` will raise an error.
 
         Raises:
-            TypeError: if one of the `cache_dependents` is not a `ZfitCachable` _and_ `allow_non_cachable`
+            TypeError: if one of the `cache_dependents` is not a `ZfitGraphCachable` _and_ `allow_non_cachable`
                 if `False`.
         """
         cache_deps = convert_to_container(cache_deps)
@@ -142,15 +147,18 @@ class GraphCachable(ZfitGraphCachable):
             if isinstance(cache_dep, ZfitGraphCachable):
                 cache_dep.register_cacher(self)
             elif not allow_non_cachable:
-                raise TypeError("cache_dependent {} is not a `ZfitCachable` but {}".format(cache_dep,
-                                                                                           type(cache_dep)))
+                raise TypeError(
+                    "cache_dependent {} is not a `ZfitGraphCachable` but {}".format(
+                        cache_dep, type(cache_dep)
+                    )
+                )
 
     def reset_cache_self(self):
         """Clear the cache of self and all dependent cachers."""
         self._clean_cache()
         self._inform_cachers()
 
-    def reset_cache(self, reseter: 'ZfitGraphCachable'):
+    def reset_cache(self, reseter: ZfitGraphCachable):
         self.reset_cache_self()
 
     def _clean_cache(self):
@@ -169,7 +177,9 @@ def invalidate_graph(func):
     def wrapped_func(*args, **kwargs):
         self = args[0]
         if not isinstance(self, ZfitGraphCachable):
-            raise TypeError("Decorator can only be used in a subclass of `ZfitCachable`")
+            raise TypeError(
+                "Decorator can only be used in a subclass of `ZfitGraphCachable`"
+            )
         self.reset_cache(reseter=self)
 
         return func(*args, **kwargs)
@@ -180,9 +190,14 @@ def invalidate_graph(func):
 class FunctionCacheHolder(GraphCachable):
     IS_TENSOR = object()
 
-    def __init__(self, func, wrapped_func,
-                 cachables: Union[ZfitGraphCachable, object, Iterable[Union[ZfitGraphCachable, object]]] = None,
-                 cachables_mapping=None):
+    def __init__(
+        self,
+        func,
+        wrapped_func,
+        cachables: (ZfitGraphCachable | object | Iterable[ZfitGraphCachable]) = None,
+        cachables_mapping=None,
+        stateless_args: bool | None = None,
+    ):
         """`tf.function` decorated function holder with caching dependencies on inputs.
 
         A `tf.function` creates a new graph for every signature that is encountered. It automatically caches them but
@@ -207,23 +222,36 @@ class FunctionCacheHolder(GraphCachable):
             cachables: objects that are cached. If they change, the cache is invalidated
             cachables_mapping: keyword arguments to the function. If the values change, the cache is
                 invalidated.
+            stateless_args: If `True`, the arguments that are normally stateful, such as `tf.Variable`s, are regarded
+                as stateless.
         """
-        # cache = {} if cache is None else cache
+
         self.delete_from_cache = False
+        if stateless_args is None:
+            stateless_args = False
+        self.stateless_args = stateless_args
         self.wrapped_func = wrapped_func
-        # self.parent_cache = cache
+
         self.python_func = func
-        self._hash_value = hash(self.python_func)
+
         if cachables is None and cachables_mapping is None:
-            raise ValueError("Both `cachables and `cachables_mapping` are None. One needs to be different from None.")
+            raise ValueError(
+                "Both `cachables and `cachables_mapping` are None. One needs to be different from None."
+            )
         if cachables is None:
             cachables = []
         if cachables_mapping is None:
             cachables_mapping = {}
+
         cachables = convert_to_container(cachables, container=list)
-        cachables_values = convert_to_container(cachables_mapping.values(), container=list)
+        cachables_values = convert_to_container(
+            cachables_mapping.values(), container=list
+        )
         cachables_all = cachables + cachables_values
-        self.immutable_representation = self.create_immutable(cachables, cachables_mapping)
+        self.immutable_representation = self.create_immutable(
+            cachables, cachables_mapping
+        )
+        self._hash_value = hash((self.python_func, self.immutable_representation))
         # self._hash_value = hash(self.immutable_representation)
         super().__init__()  # resets the cache
         self.add_cache_deps(cachables_all)
@@ -242,41 +270,65 @@ class FunctionCacheHolder(GraphCachable):
         Returns:
         """
         # is initialized before the core
-        from ..core.interfaces import ZfitData, ZfitParameter, ZfitSpace
 
         args = list(args)
         kwargs = list(kwargs.keys()) + list(kwargs.values())
-        combined = []
-        if args != []:
-            combined += args
-        if kwargs != []:
-            combined += args
+        combined = args + kwargs
         combined_cleaned = []
         for obj in combined:
-            if isinstance(obj, ZfitData):
-                obj = (id(obj),)
-
-            elif isinstance(obj, ZfitParameter):
-                obj = (ZfitParameter, obj.name)
-            elif isinstance(obj, ZfitSpace):
-                obj = (id(obj),)
-            elif tf.is_tensor(obj):
-                obj = self.IS_TENSOR
-            elif isinstance(obj, np.ndarray):
-                obj = (obj,) if sum(obj.shape) < 20 else id(obj)
-            combined_cleaned.append(obj)
-
+            obj = self.get_immutable_repr_obj(obj)
+            combined_cleaned.extend(obj)
         return tuple(combined_cleaned)
 
+    def get_immutable_repr_obj(self, obj):
+        from ..core.interfaces import ZfitData, ZfitParameter, ZfitSpace
+
+        if isinstance(obj, collections.abc.Mapping):
+            obj = obj.items()
+        if isinstance(obj, ZfitData):
+            obj = id(obj)
+        elif isinstance(obj, ZfitParameter):
+            if self.stateless_args:
+                obj = (self.IS_TENSOR,)
+            else:
+                obj = (ZfitParameter, obj.name)
+        elif isinstance(obj, ZfitSpace):
+            obj = id(obj)
+        elif tf.is_tensor(obj):
+            obj = (self.IS_TENSOR,)
+        elif isinstance(obj, np.ndarray):
+            obj = id(obj)
+        elif isinstance(obj, str):
+            obj = (obj,)
+        elif isinstance(obj, collections.abc.Iterable):
+            obj_new = []
+            for obj_i in obj:
+                obj_new.extend(self.get_immutable_repr_obj(obj_i))
+            obj = tuple(obj_new)
+        if not isinstance(obj, tuple):
+            obj = (obj,)
+        return obj
+
     def __hash__(self) -> int:
         return self._hash_value
 
     def __eq__(self, other: object) -> bool:
         if not isinstance(other, FunctionCacheHolder):
             return False
+        # return False  # HACK
+        # return self.__hash__() == other.__hash__()  # HACK TODO
         # return all(obj1 == obj2 for obj1, obj2 in zip(self.immutable_representation, other.immutable_representation))
+        array_repr_self = self.immutable_representation
+        array_repr_other = other.immutable_representation
         try:
-            return all(np.equal(self.immutable_representation, other.immutable_representation))
+            all_ids = all(
+                (obj1 is obj2) or (obj1 == obj2)
+                for obj1, obj2 in zip_longest(array_repr_self, array_repr_other)
+            )
+            # all_values = all(np.equal(array_repr_self, array_repr_other))
+            # if all_ids != all_values:
+            #     raise ValueError("Ids and values are not equal")
+            return all_ids  # TODO: this isn't optimal...
         except ValueError:  # broadcasting does not work
             return False
         except TypeError:  # OperatorNotAllowedError inherits from this
@@ -304,6 +356,6 @@ def clear_graph_cache():
     for registry in FunctionWrapperRegistry.registries:
         registry.reset()
     for instance in GraphCachable.instances:
-        instance.reset_cache('global')
+        instance.reset_cache("global")
     # Cachable.graph_caching_methods.clear()
     tf.compat.v1.reset_default_graph()
