@@ -1,4 +1,4 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
 
 # TODO: improve errors of models. Generate more general error, inherit and use more specific?
 import warnings
@@ -158,6 +158,7 @@ class ParameterNotIndependentError(Exception):
 
 # Minimizer errors
 
+
 class NotMinimizedError(Exception):
     pass
 
@@ -175,11 +176,12 @@ class CannotConvertToNumpyError(Exception):
 
 # Baseclass to steer execution
 class ZfitNotImplementedError(NotImplementedError):
-
     def __init__(self, *args: object) -> None:
         super().__init__(*args)
         if type(self) == ZfitNotImplementedError:
-            warnings.warn("Prefer to use a more specific subclass. See in `zfit.exceptions`")
+            warnings.warn(
+                "Prefer to use a more specific subclass. See in `zfit.exceptions`"
+            )
 
 
 class FunctionNotImplemented(ZfitNotImplementedError):
@@ -215,60 +217,77 @@ class AnalyticNotImplemented(ZfitNotImplementedError):
 
 class AnalyticIntegralNotImplemented(AnalyticNotImplemented):
     """If an analytic integral is not provided."""
+
     pass
 
 
 class AnalyticSamplingNotImplemented(AnalyticNotImplemented):
     """If analytic sampling from a distribution is not possible."""
+
     pass
 
 
 # PDF class internal handling errors
-class NormRangeNotImplemented(StandardControlFlow):
+class NormNotImplemented(StandardControlFlow):
     """Indicates that a function does not support the normalization range argument `norm_range`."""
+
     pass
 
 
+NormRangeNotImplemented = NormNotImplemented  # legacy
+
+
 class MultipleLimitsNotImplemented(StandardControlFlow):
     """Indicates that a function does not support several limits in a :py:class:`~zfit.Space`."""
+
     pass
 
+
 class InitNotImplemented(StandardControlFlow):
     """Indicates that a minimize method does not support a FitResult instead of a loss."""
+
     pass
 
 
 class VectorizedLimitsNotImplemented(StandardControlFlow):
     """Indicates that a function does not support vectorized (n_events > 1) limits in a :py:class:`~zfit.Space`."""
+
     pass
 
+
 class DerivativeCalculationError(ValueError):
     pass
 
+
 # Developer verbose messages
 
+
 class WorkInProgressError(Exception):
     """Only for developing purpose!
 
     Does not serve as a 'real' Exception.
     """
+
     pass
 
 
 class BreakingAPIChangeError(Exception):
     def __init__(self, msg, *args: object) -> None:
-        default_msg = ("This item has been removed due to an API change. Instruction to update:\n"
-                       "")
+        default_msg = (
+            "This item has been removed due to an API change. Instruction to update:\n"
+            ""
+        )
         msg = default_msg + str(msg)
         super().__init__(msg, *args)
 
 
 class BehaviorUnderDiscussion(Exception):
-
     def __init__(self, msg, *args: object) -> None:
-        default_msg = ("The behavior of the following is currently under discussion and ideas are well needed. "
-                       "Please open an issue at https://github.com/zfit/zfit/issues with your opinion about this.\n"
-                       "")
+        default_msg = (
+            "The behavior of the following is currently under discussion and ideas are well needed. "
+            "Please open an issue at https://github.com/zfit/zfit/issues with your opinion about this.\n"
+            ""
+        )
         msg = default_msg + str(msg)
         super().__init__(msg, *args)
 
