@@ -5,6 +5,19 @@ numpy api of tensorflow. See https://www.tensorflow.org/guide/tf_numpy for more
 https://www.tensorflow.org/api_docs/python/tf/experimental/numpy for the complete numpy api in tensorflow. Recommended
 way of importing:
 
->>> Import zfit.z.numpy as znp
+>>> import zfit.z.numpy as znp
 """
+#  Copyright (c) 2022 zfit
+
 from tensorflow.experimental.numpy import *
+import tensorflow as tf
+
+
+class linalg:
+
+    inv = staticmethod(tf.linalg.inv)
+    det = staticmethod(tf.linalg.det)
+    solve = staticmethod(tf.linalg.solve)
+
+
+del tf
