@@ -1,14 +1,18 @@
-#  Copyright (c) 2021 zfit
+#  Copyright (c) 2022 zfit
+
+from __future__ import annotations
+
 import functools
+import logging
 import math as _mt
 from collections import defaultdict
-from typing import Any, Callable
+from collections.abc import Callable
+from typing import Any
 
 import numpy as np
 import tensorflow as tf
 
 import zfit.z.numpy as znp
-
 from ..settings import ztypes
 from ..util.exception import BreakingAPIChangeError
 from ..util.warnings import warn_advanced_feature
@@ -17,7 +21,9 @@ from ..util.warnings import warn_advanced_feature
 def constant(value, dtype=ztypes.float, shape=None, name="Const", verify_shape=None):
     # TODO(tf2): remove this legacy thing below
     if verify_shape is not None:
-        raise RuntimeError("'verify_shape' is not a valid argument anymore. It's always true. Please remove.")
+        raise RuntimeError(
+            "'verify_shape' is not a valid argument anymore. It's always true. Please remove."
+        )
     return tf.constant(value, dtype=dtype, shape=shape, name=name)
 
 
@@ -29,14 +35,14 @@ def to_complex(number, dtype=ztypes.complex):
 
 
 def to_real(x, dtype=ztypes.float):
-    return tf.cast(x, dtype=dtype)
+    return znp.asarray(tf.cast(x, dtype=dtype))
 
 
 def abs_square(x):
     return znp.real(x * znp.conj(x))
 
 
-def nth_pow(x, n, name=None):
+def nth_pow(x, n):
     """Calculate the nth power of the complex Tensor x.
 
     Args:
@@ -44,16 +50,22 @@ def nth_pow(x, n, name=None):
         n: Power of x, has to be a positive int
         name: No effect, for API compatibility with tf.pow
     """
-    if not n >= 0:
+    if n < 0:
         raise ValueError(f"n (power) has to be >= 0. Currently, n={n}")
 
-    power = to_complex(1.)
+    power = to_complex(1.0)
     for _ in range(n):
         power *= x
     return power
 
 
-def unstack_x(value: Any, num: Any = None, axis: int = -1, always_list: bool = False, name: str = "unstack_x"):
+def unstack_x(
+    value: Any,
+    num: Any = None,
+    axis: int = -1,
+    always_list: bool = False,
+    name: str = "unstack_x",
+):
     """Unstack a Data object and return a list of (or a single) tensors in the right order.
 
     Args:
@@ -87,11 +99,18 @@ def stack_x(values, axis: int = -1, name: str = "stack_x"):
 
 
 def convert_to_tensor(value, dtype=None, name=None, preferred_dtype=None):
-    return tf.convert_to_tensor(value=value, dtype=dtype, name=name, dtype_hint=preferred_dtype)
+    return tf.convert_to_tensor(
+        value=value, dtype=dtype, name=name, dtype_hint=preferred_dtype
+    )
 
 
-def safe_where(condition: tf.Tensor, func: Callable, safe_func: Callable, values: tf.Tensor,
-               value_safer: Callable = tf.ones_like) -> tf.Tensor:
+def safe_where(
+    condition: tf.Tensor,
+    func: Callable,
+    safe_func: Callable,
+    values: tf.Tensor,
+    value_safer: Callable = tf.ones_like,
+) -> tf.Tensor:
     """Like :py:func:`tf.where` but fixes gradient `NaN` if func produces `NaN` with certain `values`.
 
     Args:
@@ -119,14 +138,19 @@ def run_no_nan(func, x):
 
     value_with_nans = func(x=x)
     if value_with_nans.dtype in (tf.complex128, tf.complex64):
-        value_with_nans = znp.real(value_with_nans) + znp.imag(value_with_nans)  # we care only about NaN or not
+        value_with_nans = znp.real(value_with_nans) + znp.imag(
+            value_with_nans
+        )  # we care only about NaN or not
     finite_bools = znp.isfinite(tf.cast(value_with_nans, dtype=tf.float64))
     finite_indices = tf.where(finite_bools)
     new_x = tf.gather_nd(params=x, indices=finite_indices)
     new_x = Data.from_tensor(obs=x.obs, tensor=new_x)
     vals_no_nan = func(x=new_x)
-    result = tf.scatter_nd(indices=finite_indices, updates=vals_no_nan,
-                           shape=tf.shape(input=value_with_nans, out_type=finite_indices.dtype))
+    result = tf.scatter_nd(
+        indices=finite_indices,
+        updates=vals_no_nan,
+        shape=tf.shape(input=value_with_nans, out_type=finite_indices.dtype),
+    )
     return result
 
 
@@ -135,29 +159,35 @@ class FunctionWrapperRegistry:
     registries = []
     allow_jit = True
     _DEFAULT_DO_JIT_TYPES = defaultdict(lambda: True)
-    _DEFAULT_DO_JIT_TYPES.update({
-        None: True,
-        'model': False,
-        'loss': True,
-        'sample': True,
-        'model_sampling': True,
-        'zfit_tensor': True,
-        'tensor': True,
-    })
+    _DEFAULT_DO_JIT_TYPES.update(
+        {
+            None: True,
+            "model": False,
+            "loss": True,
+            "sample": True,
+            "model_sampling": True,
+            "zfit_tensor": True,
+            "tensor": True,
+        }
+    )
 
     do_jit_types = _DEFAULT_DO_JIT_TYPES.copy()
 
     @classmethod
     def all_wrapped_functions_registered(cls):
-        return all(func.zfit_graph_cache_registered for func in cls.all_wrapped_functions)
+        return all(
+            func.zfit_graph_cache_registered for func in cls.all_wrapped_functions
+        )
 
-    def __init__(self, wraps=None, **kwargs_user) -> None:
+    def __init__(self, wraps=None, stateless_args=None, **kwargs_user) -> None:
         """`tf.function`-like decorator with additional cache-invalidation functionality.
 
         Args:
             **kwargs_user: arguments to `tf.function`
         """
         super().__init__()
+        if stateless_args is None:
+            stateless_args = False
         self._initial_user_kwargs = kwargs_user
 
         self.registries.append(self)
@@ -167,6 +197,7 @@ class FunctionWrapperRegistry:
             # raise RuntimeError(f"Currently custom 'wraps' category ({wraps}) not allowed, set explicitly in `do_jit_types`")
             self.do_jit_types[wraps] = True
         self.wraps = wraps
+        self.stateless_args = stateless_args
         self.function_cache = defaultdict(list)
         self.reset(**self._initial_user_kwargs)
         self.currently_traced = set()
@@ -176,10 +207,9 @@ class FunctionWrapperRegistry:
         return self.do_jit_types[self.wraps] and self.allow_jit
 
     def reset(self, **kwargs_user):
-        kwargs = dict(autograph=False, experimental_relax_shapes=True)
+        kwargs = dict(autograph=False, reduce_retracing=False)
         kwargs.update(self._initial_user_kwargs)
         kwargs.update(kwargs_user)
-
         self.tf_function = tf.function(**kwargs)
         for cache in self.function_cache.values():
             cache.clear()
@@ -201,15 +231,21 @@ class FunctionWrapperRegistry:
             function_holder = FunctionCacheHolder(func, wrapped_func, args, kwargs)
 
             if function_holder in cache:
+                # print("DEBUG: using cached function")
                 func_holder_index = cache.index(function_holder)
                 func_holder_cached = cache[func_holder_index]
                 if func_holder_cached.is_valid:
                     function_holder = func_holder_cached
                 else:
-                    wrapped_func = self.tf_function(func)  # update nonlocal wrapped function
-                    function_holder = FunctionCacheHolder(func, wrapped_func, args, kwargs)
+                    wrapped_func = self.tf_function(
+                        func
+                    )  # update nonlocal wrapped function
+                    function_holder = FunctionCacheHolder(
+                        func, wrapped_func, args, kwargs
+                    )
                     cache[func_holder_index] = function_holder
             else:
+                # print(f"DEBUG: caching function {func} with args {args} and kwargs {kwargs}")
                 cache.append(function_holder)
             func_to_run = function_holder.wrapped_func
             try:
@@ -223,35 +259,56 @@ class FunctionWrapperRegistry:
 
 
 # equivalent to tf.function
-def function(func=None, **kwargs):
+def function(func=None, *, stateless_args=None, **kwargs):
+    """JIT/Graph compilation of functions, `tf.function`-like with additional cache-invalidation functionality.
+
+    Args:
+        func: Function to be compiled.
+        stateless_args: If True, the function is assumed to be stateless and *does not depend on the name of tf.Variables*
+            but only needs the values of the variables. This is not the case for taking gradients, for example.
+        **kwargs: arguments to `tf.function`
+
+    Returns:
+    """
+    if stateless_args is None:
+        stateless_args = False
     if callable(func):
         wrapper = FunctionWrapperRegistry()
         return wrapper(func)
     elif func:
-        raise ValueError("All argument have to be key-word only. `func` must not be used")
+        raise ValueError(
+            "All argument have to be key-word only. `func` must not be used"
+        )
     else:
-        return FunctionWrapperRegistry(**kwargs)
+        return FunctionWrapperRegistry(**kwargs, stateless_args=stateless_args)
 
 
 # legacy, remove 0.6
 def function_tf_input(*_, **__):
-    raise BreakingAPIChangeError("This function has been removed. Use `z.function(wraps='zfit_tensor') or your"
-                                 "own category")
+    raise BreakingAPIChangeError(
+        "This function has been removed. Use `z.function(wraps='zfit_tensor') or your"
+        "own category"
+    )
 
 
 # legacy, remove 0.6
 def function_sampling(*_, **__):
-    raise BreakingAPIChangeError("This function has been removed. Use `z.function(wraps='zfit_sampling') or your"
-                                 "own category")
+    raise BreakingAPIChangeError(
+        "This function has been removed. Use `z.function(wraps='zfit_sampling') or your"
+        "own category"
+    )
 
 
 @functools.wraps(tf.py_function)
 def py_function(func, inp, Tout, name=None):
     from .. import settings
-    if not settings.options['numerical_grad']:
-        warn_advanced_feature("Using py_function without numerical gradients. If the Python code does not contain any"
-                              " parametrization by `zfit.Parameter` or similar, this can work out. Otherwise, in case"
-                              " it depends on those, you may want to set `zfit.run.set_autograd_mode(=False)`.",
-                              identifier="py_func_autograd")
+
+    if not settings.options["numerical_grad"]:
+        warn_advanced_feature(
+            "Using py_function without numerical gradients. If the Python code does not contain any"
+            " parametrization by `zfit.Parameter` or similar, this can work out. Otherwise, in case"
+            " it depends on those, you may want to set `zfit.run.set_autograd_mode(=False)`.",
+            identifier="py_func_autograd",
+        )
 
     return tf.py_function(func=func, inp=inp, Tout=Tout, name=name)
